@@ -25,7 +25,7 @@
  */
 
 /*
- * Modified by Marcus Bjurman <marbj499@student.liu.se> 2001-2003
+ * Modified by Marcus Bjurman <marbj499@student.liu.se> 2001-2006
  * The orginal comments are left intact above
  */
 
@@ -121,17 +121,17 @@ get_pos (GnomeCmdCombo *combo, gint *x, gint *y, gint *height, gint *width)
 		list_requisition.height += EMPTY_LIST_HEIGHT;
   
 	alloc_width = (widget->allocation.width -
-				   2 * popwin->child->style->klass->xthickness -
+				   2 * popwin->child->style->xthickness -
 				   2 * GTK_CONTAINER (popwin->child)->border_width -
 				   2 * GTK_CONTAINER (combo->popup)->border_width -
 				   2 * GTK_CONTAINER (GTK_BIN (popup)->child)->border_width - 
-				   2 * GTK_BIN (popup)->child->style->klass->xthickness) + 100;
+				   2 * GTK_BIN (popup)->child->style->xthickness) + 100;
   
-	work_height = (2 * popwin->child->style->klass->ythickness +
+	work_height = (2 * popwin->child->style->ythickness +
 				   2 * GTK_CONTAINER (popwin->child)->border_width +
 				   2 * GTK_CONTAINER (combo->popup)->border_width +
 				   2 * GTK_CONTAINER (GTK_BIN (popup)->child)->border_width +
-				   2 * GTK_BIN (popup)->child->style->klass->xthickness)+20;
+				   2 * GTK_BIN (popup)->child->style->xthickness)+20;
   
 	do 
     {
@@ -142,8 +142,8 @@ get_pos (GnomeCmdCombo *combo, gint *x, gint *y, gint *height, gint *width)
 			alloc_width < list_requisition.width)
 		{
 			work_height += popup->hscrollbar->requisition.height +
-				GTK_SCROLLED_WINDOW_CLASS 
-				(GTK_OBJECT (combo->popup)->klass)->scrollbar_spacing;
+				GTK_SCROLLED_WINDOW_GET_CLASS 
+				(combo->popup)->scrollbar_spacing;
 			show_hscroll = TRUE;
 		}
 		if (!show_vscroll && 
@@ -157,8 +157,8 @@ get_pos (GnomeCmdCombo *combo, gint *x, gint *y, gint *height, gint *width)
 			}
 			alloc_width -= 
 				popup->vscrollbar->requisition.width +
-				GTK_SCROLLED_WINDOW_CLASS 
-				(GTK_OBJECT (combo->popup)->klass)->scrollbar_spacing;
+				GTK_SCROLLED_WINDOW_GET_CLASS 
+				(combo->popup)->scrollbar_spacing;
 			show_vscroll = TRUE;
 		}
     } while (old_width != alloc_width || old_height != work_height);
@@ -189,13 +189,6 @@ gnome_cmd_combo_popup_list (GnomeCmdCombo *combo)
 	gtk_widget_realize (combo->popwin);
 	gdk_window_resize (combo->popwin->window, width, height);
 	gtk_widget_show (combo->popwin);
-
-	if (combo->first_popup) {
-		gtk_clist_unselect_all (GTK_CLIST (combo->list));
-		GTK_CLIST (combo->list)->focus_row = 0;
-		gtk_clist_select_row (GTK_CLIST (combo->list), 0, 0);
-		combo->first_popup = FALSE;
-	}
 }
 
 
@@ -206,58 +199,17 @@ gnome_cmd_combo_popup_list (GnomeCmdCombo *combo)
  * Callbacks
  *******************************/
 
-static gboolean        
-on_popup_button_press (GtkWidget        *button,
-					   GdkEventButton   *event,
-					   GnomeCmdCombo    *combo)
-{
-	GtkWidget *child;
-
-	if (!event) return FALSE;
-	if (event->button != 1) return FALSE;
-
-	// Check to see if we released inside the button
-	child = gtk_get_event_widget ((GdkEvent*) event);
-	
-	while (child && child != (combo->button))
-		child = child->parent;
-	
-	if (child == combo->button) {
-		return FALSE;
-	}
-
-//	gtk_widget_hide (combo->popwin);
-//	combo->is_popped = FALSE;
-	return TRUE;
-}
-
-
 static gboolean        
 on_popup_button_release (GtkWidget        *button,
-						 GdkEventButton   *event,
 						 GnomeCmdCombo    *combo)
 {
-	GtkWidget *child;
-	
-	if (!event) return FALSE;
-	if (event->button != 1) return FALSE;
-
-	// Check to see if we released inside the button
-	child = gtk_get_event_widget ((GdkEvent*) event);
-	
-	while (child && child != (combo->button))
-		child = child->parent;
-	
-	if (child == combo->button) {
-		if (combo->is_popped) {
-			gtk_widget_hide (combo->popwin);
-			combo->is_popped = FALSE;
-		}
-		else {
-			gnome_cmd_combo_popup_list (combo);
-			combo->is_popped = TRUE;
-		}
-		return FALSE;
+	if (combo->is_popped) {
+		gtk_widget_hide (combo->popwin);
+		combo->is_popped = FALSE;
+	}
+	else {
+		gnome_cmd_combo_popup_list (combo);
+		combo->is_popped = TRUE;
 	}
 
 	return TRUE;
@@ -323,18 +275,44 @@ on_popwin_keypress (GtkWidget *widget, GdkEventKey *event, GnomeCmdCombo *combo)
 }
 
 
-static void
-on_list_select_row (GtkCList *list, gint row, gint col,
-					GdkEventButton *event, GnomeCmdCombo *combo)
+static gboolean
+on_list_button_press (GtkCList *clist, GdkEventButton *event, GnomeCmdCombo *combo)
 {
-	GdkEventKey event2;
-	if (!event) return;
+	gint row;
 
-	event2.type = GDK_KEY_PRESS;
-	event2.keyval = GDK_Return;
-	event2.state = 0;
+	if (clist->clist_window != event->window)
+		return FALSE;
 
-	on_popwin_keypress (GTK_WIDGET (combo->popwin), &event2, combo);
+	row = gnome_cmd_clist_get_row (GNOME_CMD_CLIST (clist), event->x, event->y);
+	if (row < 0)
+		return FALSE;
+
+	if (row != clist->focus_row) {
+		gtk_clist_select_row (clist, row, 0);
+		clist->focus_row = row;
+	}
+	else
+		gtk_signal_emit_stop_by_name (GTK_OBJECT (clist), "button-press-event");
+	
+	return TRUE;
+}
+
+
+static gboolean
+on_list_button_release (GtkCList *clist, GdkEventButton *event, GnomeCmdCombo *combo)
+{
+	gint row;
+	
+	row = gnome_cmd_clist_get_row (GNOME_CMD_CLIST (clist), event->x, event->y);
+	
+	if (clist->focus_row >= 0 && clist->focus_row == row) {
+		gpointer data = gtk_clist_get_row_data (clist, clist->focus_row);
+		gtk_signal_emit (GTK_OBJECT (combo), combo_signals[ITEM_SELECTED], data);
+
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 
@@ -381,7 +359,7 @@ class_init (GnomeCmdComboClass *klass)
 	combo_signals[ITEM_SELECTED] =
 		gtk_signal_new ("item_selected",
 			GTK_RUN_LAST,
-		    object_class->type,
+		    G_OBJECT_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GnomeCmdComboClass, item_selected),
 		    gtk_marshal_NONE__POINTER,
 		    GTK_TYPE_NONE,
@@ -390,14 +368,12 @@ class_init (GnomeCmdComboClass *klass)
 	combo_signals[POPWIN_HIDDEN] =
 		gtk_signal_new ("popwin_hidden",
 			GTK_RUN_LAST,
-		    object_class->type,
+		    G_OBJECT_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GnomeCmdComboClass, popwin_hidden),
 		    gtk_marshal_NONE__NONE,
 		    GTK_TYPE_NONE,
 			0);
 	
-	gtk_object_class_add_signals (object_class, combo_signals, LAST_SIGNAL);
-	
 	object_class->destroy = destroy;  
 	widget_class->size_allocate = size_allocate;
 	klass->item_selected = gnome_cmd_combo_item_selected;
@@ -423,9 +399,10 @@ init (GnomeCmdCombo *combo)
 	gtk_object_set_data_full (GTK_OBJECT (combo),
 							  "entry", combo->entry,
 							  (GtkDestroyNotify) gtk_widget_unref);
-//	gtk_widget_set_style (combo->entry, list_style);
 	gtk_widget_show (combo->entry);
 	gtk_widget_set_usize (combo->entry, 60, -1);
+	gtk_entry_set_editable (GTK_ENTRY (combo->entry), FALSE);
+	GTK_WIDGET_UNSET_FLAGS (combo->entry, GTK_CAN_FOCUS);
 	
 	combo->button = gtk_button_new ();
 	gtk_widget_ref (combo->button);
@@ -445,15 +422,11 @@ init (GnomeCmdCombo *combo)
 	gtk_container_add (GTK_CONTAINER (combo->button), arrow);
 	gtk_box_pack_start (GTK_BOX (combo), combo->entry, TRUE, TRUE, 0);
 	gtk_box_pack_end (GTK_BOX (combo), combo->button, FALSE, FALSE, 0);
-	GTK_WIDGET_UNSET_FLAGS (combo->button, GTK_CAN_FOCUS);
 
 	/* connect button signals */
-	gtk_signal_connect_after (
-		GTK_OBJECT (combo->button), "button-release-event",
+	gtk_signal_connect (
+		GTK_OBJECT (combo->button), "clicked",
 		(GtkSignalFunc) on_popup_button_release, combo);
-	gtk_signal_connect_after (
-		GTK_OBJECT (combo->button), "button-press-event",
-		(GtkSignalFunc) on_popup_button_press, combo);
 	
 	combo->popwin = gtk_window_new (GTK_WINDOW_POPUP);
 	gtk_widget_ref (combo->popwin);
@@ -544,17 +517,19 @@ gnome_cmd_combo_get_type (void)
 }
 
 GtkWidget *
-gnome_cmd_combo_new (gint num_cols, gchar **col_titles)
+gnome_cmd_combo_new (gint num_cols, gint text_col, gchar **col_titles)
 {
 	GnomeCmdCombo *combo =  gtk_type_new (gnome_cmd_combo_get_type ());
 
+	combo->text_col = text_col;
+	combo->sel_data = NULL;
+	combo->sel_text = NULL;
+	
 	if (col_titles)
 		combo->list = gnome_cmd_clist_new_with_titles (num_cols, col_titles);
 	else
 		combo->list = gnome_cmd_clist_new (num_cols);
 
-	combo->first_popup = TRUE;
-	
 	gtk_widget_ref (combo->list);
 	gtk_object_set_data_full (GTK_OBJECT (combo),
 							  "combo->list", combo->list,
@@ -567,9 +542,12 @@ gnome_cmd_combo_new (gint num_cols, gchar **col_titles)
 	gtk_widget_show (combo->list);
 
 	/* connect list signals */
-	gtk_signal_connect_after (
-		GTK_OBJECT (combo->list), "select-row",
-		GTK_SIGNAL_FUNC (on_list_select_row), combo);
+	gtk_signal_connect (
+		GTK_OBJECT (combo->list), "button-press-event",
+		GTK_SIGNAL_FUNC (on_list_button_press), combo);
+	gtk_signal_connect (
+		GTK_OBJECT (combo->list), "button-release-event",
+		GTK_SIGNAL_FUNC (on_list_button_release), combo);
 	gtk_signal_connect (
 		GTK_OBJECT (combo->list), "key-press-event",
 		GTK_SIGNAL_FUNC (on_list_key_press), combo);
@@ -595,6 +573,7 @@ gnome_cmd_combo_append (GnomeCmdCombo *combo, gchar **text, gpointer data)
 
 	row = gtk_clist_append (GTK_CLIST (combo->list), text);
 	gtk_clist_set_row_data (GTK_CLIST (combo->list), row, data);
+
 	return row;
 }
 
@@ -609,6 +588,7 @@ gnome_cmd_combo_insert (GnomeCmdCombo *combo, gchar **text, gpointer data)
 
 	row = gtk_clist_insert (GTK_CLIST (combo->list), 0, text);
 	gtk_clist_set_row_data (GTK_CLIST (combo->list), row, data);
+	
 	return row;
 }
 
@@ -635,7 +615,7 @@ gnome_cmd_combo_set_pixmap (GnomeCmdCombo *combo,
 
 
 void
-gnome_cmd_combo_set_selection (GnomeCmdCombo *combo, const gchar *text)
+gnome_cmd_combo_select_text (GnomeCmdCombo *combo, const gchar *text)
 {
 	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
 	g_return_if_fail (text != NULL);
@@ -644,37 +624,67 @@ gnome_cmd_combo_set_selection (GnomeCmdCombo *combo, const gchar *text)
 }
 
 
+void
+gnome_cmd_combo_select_data (GnomeCmdCombo *combo,
+							 gpointer data)
+{
+	gint row;
+	GtkCList *clist;
+
+	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
+
+	clist = GTK_CLIST (combo->list);
+	row = gtk_clist_find_row_from_data (clist, data);
+
+	if (gtk_clist_get_text (clist, row, combo->text_col, &combo->sel_text)) {
+		combo->sel_data = data;
+		gtk_entry_set_text (GTK_ENTRY (combo->entry), combo->sel_text);
+		gtk_clist_select_row (GTK_CLIST (combo->list), row, 0);
+	}
+}
+
+
 void
 gnome_cmd_combo_update_style (GnomeCmdCombo *combo)
 {
-	gint i;
-	
 	gtk_widget_set_style (combo->entry, list_style);
 	gnome_cmd_clist_update_style (GNOME_CMD_CLIST (combo->list));
-
-	for ( i=0 ; i<GTK_CLIST (combo->list)->columns ; i++ ) {
-		GtkWidget *button;
-		GtkWidget *child;
-
-		button = GTK_CLIST (combo->list)->column[i].button;
-		if (!button) return;
-
-		child = GTK_BIN (button)->child;
-		if (!child) return;
-
-		child = GTK_BIN (child)->child;
-		if (!child) return;
-		
-		gtk_widget_modify_style (child, main_rc_style);
-	}
 }
 
 
 static void
 gnome_cmd_combo_item_selected (GnomeCmdCombo *combo, gpointer data)
 {
+	gint row;
+	
 	g_return_if_fail (GNOME_CMD_IS_COMBO (combo));
 
 	gtk_widget_hide (combo->popwin);
 	combo->is_popped = FALSE;
+
+	row = gtk_clist_find_row_from_data (GTK_CLIST (combo->list), data);
+
+	if (gtk_clist_get_text (GTK_CLIST (combo->list), row, combo->text_col, &combo->sel_text)) {
+		combo->sel_data = data;
+		gtk_entry_set_text (GTK_ENTRY (combo->entry), combo->sel_text);
+	}
 }
+
+
+gpointer
+gnome_cmd_combo_get_selected_data (GnomeCmdCombo *combo)
+{
+	g_return_val_if_fail (GNOME_CMD_IS_COMBO (combo), NULL);
+
+	return combo->sel_data;
+}
+
+
+const gchar*
+gnome_cmd_combo_get_selected_text (GnomeCmdCombo *combo)
+{
+	g_return_val_if_fail (GNOME_CMD_IS_COMBO (combo), NULL);
+	
+	return combo->sel_text;
+}
+
