@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2004 Marcus Bjurman
+    Copyright (C) 2001-2006 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,6 +31,8 @@ struct _GnomeCmdConPrivate
 	History        *dir_history;
 	GnomeCmdBookmarkGroup *bookmarks;
 	GnomeCmdDirPool *dir_pool;
+	GList          *all_dirs;
+	GHashTable     *all_dirs_map;
 };
 
 enum {
@@ -196,6 +198,8 @@ init (GnomeCmdCon *con)
 	con->priv->bookmarks->bookmarks = NULL;
 	con->priv->bookmarks->con = con;
 	con->priv->bookmarks->data = NULL;
+	con->priv->all_dirs = NULL;
+	con->priv->all_dirs_map = NULL;
 }
 
 
@@ -239,7 +243,7 @@ check_con_open_progress (GnomeCmdCon *con)
 		GnomeCmdDir *dir;
 		
 		DEBUG('m', "CON_OPEN_OK detected\n");
-		
+
 		dir = gnome_cmd_dir_new_with_con (
 			con->base_info, con->base_path, con);
 
@@ -390,6 +394,10 @@ gnome_cmd_con_set_cwd (GnomeCmdCon *con, GnomeCmdDir *dir)
 {
 	g_return_if_fail (GNOME_CMD_IS_CON (con));
 
+	if (GNOME_CMD_IS_DIR (dir) &&
+		gnome_cmd_dir_get_connection (dir) != con)
+		return;
+
 	if (dir == con->priv->cwd)
 		return;
 
@@ -698,3 +706,58 @@ gnome_cmd_con_mkdir (GnomeCmdCon *con, const gchar *path_str)
 }
 
 
+void
+gnome_cmd_con_add_to_cache (GnomeCmdCon *con, GnomeCmdDir *dir)
+{
+	gchar *uri_str;
+
+	g_return_if_fail (GNOME_CMD_IS_CON (con));
+	g_return_if_fail (GNOME_CMD_IS_DIR (dir));
+	
+	uri_str = gnome_cmd_file_get_uri_str (GNOME_CMD_FILE (dir));
+	
+	if (!con->priv->all_dirs_map)
+		con->priv->all_dirs_map = g_hash_table_new_full (
+			g_str_hash, g_str_equal, g_free, NULL);
+
+	DEBUG ('p', "ADDING 0x%x %s to the cache\n", (guint)dir, uri_str);
+	g_hash_table_insert (con->priv->all_dirs_map, uri_str, dir);
+}
+
+
+void
+gnome_cmd_con_remove_from_cache (GnomeCmdCon *con, GnomeCmdDir *dir)
+{
+	gchar *uri_str;
+	
+	g_return_if_fail (GNOME_CMD_IS_CON (con));
+	g_return_if_fail (GNOME_CMD_IS_DIR (dir));
+	
+	uri_str = gnome_cmd_file_get_uri_str (GNOME_CMD_FILE (dir));
+	
+	DEBUG ('p', "REMOVING 0x%x %s from the cache\n", (guint)dir, uri_str);
+	g_hash_table_remove (con->priv->all_dirs_map, uri_str);
+	g_free (uri_str);
+}
+
+
+GnomeCmdDir *
+gnome_cmd_con_cache_lookup (GnomeCmdCon *con, const gchar *uri_str)
+{
+	GnomeCmdDir *dir = NULL;
+	
+	g_return_val_if_fail (GNOME_CMD_IS_CON (con), NULL);
+	g_return_val_if_fail (uri_str != NULL, NULL);
+	
+	if (con->priv->all_dirs_map)
+		dir = g_hash_table_lookup (con->priv->all_dirs_map, uri_str);
+	
+	if (dir) {
+		DEBUG ('p', "FOUND 0x%x %s in the hash-table, reusing it!\n", (guint)dir, uri_str);
+		return dir;
+	}
+	
+	DEBUG ('p', "FAILED to find %s in the hash-table\n", uri_str);
+	return NULL;
+}
+
