@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2004 Marcus Bjurman
+    Copyright (C) 2001-2006 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -645,6 +645,26 @@ gnome_cmd_dir_set_path (GnomeCmdDir *dir, GnomeCmdPath *path)
 }
 
 
+void
+gnome_cmd_dir_update_path (GnomeCmdDir *dir)
+{
+	GnomeCmdPath *path;
+	GnomeCmdDir *parent;
+	
+	g_return_if_fail (GNOME_CMD_IS_DIR (dir));
+
+	parent = gnome_cmd_dir_get_parent (dir);
+	if (!parent)
+		return;
+	
+	path = gnome_cmd_path_get_child (
+		gnome_cmd_dir_get_path (parent),
+		gnome_cmd_file_get_name (GNOME_CMD_FILE (dir)));
+	if (path)
+		gnome_cmd_dir_set_path (dir, path);
+}
+
+
 gchar *
 gnome_cmd_dir_get_display_path (GnomeCmdDir *dir)
 {
