@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2003 Marcus Bjurman
+    Copyright (C) 2001-2006 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include "gnome-cmd-delete-dialog.h"
 #include "gnome-cmd-advrename-dialog.h"
 #include "gnome-cmd-quicksearch-popup.h"
+#include "gnome-cmd-file-collection.h"
 #include "ls_colors.h"
 
 /* Controlls if file-uris should be escaped for local files when drag-N-dropping
@@ -46,7 +47,7 @@
 /* The time (in ms) it takes from that the right mouse button is clicked on a file until 
  * the popup menu appears when the right btn is used to select files.
  */
-#define POPUP_TIMEOUT 1000
+#define POPUP_TIMEOUT 750
 
 
 enum {
@@ -58,7 +59,7 @@ enum {
 };
 
 
-static GtkTargetEntry drag_types [] = {
+GtkTargetEntry drag_types [] = {
 	{ TARGET_URI_LIST_TYPE, 0, TARGET_URI_LIST },
 	{ TARGET_TEXT_PLAIN_TYPE, 0, TARGET_TEXT_PLAIN },
 	{ TARGET_URL_TYPE, 0, TARGET_URL }
@@ -103,6 +104,30 @@ GtkJustification list_column_justification[FILE_LIST_NUM_COLUMNS] = {
 };
 
 
+static gint sort_by_name (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_extension (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_dir (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_size (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_perm (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_date (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_owner (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+static gint sort_by_group (GnomeCmdFile *f1, GnomeCmdFile *f2, GnomeCmdFileList *fl);
+
+
+static GnomeVFSListCompareFunc sort_funcs[] =
+{
+	NULL,
+	(GnomeVFSListCompareFunc)sort_by_name,
+	(GnomeVFSListCompareFunc)sort_by_extension,
+	(GnomeVFSListCompareFunc)sort_by_dir,
+	(GnomeVFSListCompareFunc)sort_by_size,
+	(GnomeVFSListCompareFunc)sort_by_date,
+	(GnomeVFSListCompareFunc)sort_by_perm,
+	(GnomeVFSListCompareFunc)sort_by_owner,
+	(GnomeVFSListCompareFunc)sort_by_group
+};
+
+
 struct _GnomeCmdFileListPrivate {
 	GtkWidget *column_pixmaps[FILE_LIST_NUM_COLUMNS];
 	GtkWidget *column_labels[FILE_LIST_NUM_COLUMNS];
@@ -111,7 +136,7 @@ struct _GnomeCmdFileListPrivate {
 	GnomeVFSListCompareFunc sort_func;
 	gint current_col;
 	gboolean sort_raising[FILE_LIST_NUM_COLUMNS];
-	GList *shown_files;    /* contains GnomeCmdFile pointers */
+	GnomeCmdFileCollection *shown_files;
 	GList *selected_files; /* contains GnomeCmdFile pointers */
 	gint cur_file;
 	gboolean shift_down;
@@ -121,6 +146,7 @@ struct _GnomeCmdFileListPrivate {
 	guint right_mb_timeout_id;
 	GtkWidget *selpat_dialog;
 	GtkWidget *quicksearch_popup;
+	gchar *focus_later;
 };
 
 
@@ -293,7 +319,7 @@ toggle_file (GnomeCmdFileList *fl,
 	if (row == -1)
 		return;
 	
-	if (row < g_list_length (fl->priv->shown_files))
+	if (row < gnome_cmd_file_collection_get_size (fl->priv->shown_files))
 	{
 		if (g_list_index (fl->priv->selected_files, finfo) == -1)
 			select_file (fl, finfo);
@@ -983,37 +1009,9 @@ on_column_clicked                        (GtkCList *list,
 {
 	g_return_if_fail (GTK_IS_CLIST (list));
 	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
-	
-	switch (col)
-	{
-		case 1:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_name;
-			break;
-		case 2:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_extension;
-			break;
-		case 3:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_dir;
-			break;
-		case 4:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_size;
-			break;
-		case 5:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_date;
-			break;
-		case 6:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_perm;
-			break;
-		case 7:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_owner;
-			break;
-		case 8:
-			fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_group;
-			break;
-		default:
-			return;
-	}
 
+	fl->priv->sort_func = sort_funcs[col];
+	
 	if (fl->priv->current_col == col)
 		fl->priv->sort_raising[col] = !fl->priv->sort_raising[col];
 
@@ -1021,6 +1019,8 @@ on_column_clicked                        (GtkCList *list,
 	update_column_sort_arrows (fl);
 	
 	gnome_cmd_file_list_sort (fl);
+
+	gnome_cmd_data_set_sort_params (fl, col, fl->priv->sort_raising[col]);
 }
 
 
@@ -1241,7 +1241,7 @@ destroy (GtkObject *object)
 {
 	GnomeCmdFileList *fl = GNOME_CMD_FILE_LIST (object);
 	
-	gnome_cmd_file_list_free (fl->priv->shown_files);
+	gtk_object_destroy (GTK_OBJECT (fl->priv->shown_files));
 	gnome_cmd_file_list_free (fl->priv->selected_files);
 
 	if (fl->priv)
@@ -1316,17 +1316,25 @@ static void
 init (GnomeCmdFileList *fl)
 {
 	gint i;
+	gboolean b;
 	
 	fl->priv = g_new (GnomeCmdFileListPrivate, 1);
-	fl->priv->shown_files = NULL;
+	fl->priv->shown_files = gnome_cmd_file_collection_new ();
 	fl->priv->selected_files = NULL;
 	fl->priv->shift_down = FALSE;
-	fl->priv->sort_func = (GnomeVFSListCompareFunc)sort_by_name;
-	fl->priv->current_col = 1;
 	fl->priv->selpat_dialog = NULL;
 	fl->priv->right_mb_down_file = NULL;
 	fl->priv->right_mb_timeout_id = 0;
 	fl->priv->quicksearch_popup = NULL;
+	fl->priv->focus_later = NULL;
+
+	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ )
+		fl->priv->sort_raising[i] = FALSE;
+	
+	gnome_cmd_data_get_sort_params (fl, &i, &b);
+	fl->priv->current_col = i;
+	fl->priv->sort_raising[i] = b;
+	fl->priv->sort_func = sort_funcs[i];
 	
 	list_column_titles[FILE_LIST_COLUMN_ICON]  = "";
 	list_column_titles[FILE_LIST_COLUMN_NAME]  = _("name");
@@ -1338,9 +1346,6 @@ init (GnomeCmdFileList *fl)
 	list_column_titles[FILE_LIST_COLUMN_OWNER] = _("uid");
 	list_column_titles[FILE_LIST_COLUMN_GROUP] = _("gid");
 	
-	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ )
-		fl->priv->sort_raising[i] = FALSE;
-
 	init_dnd (fl);
 	
 	for ( i=0 ; i<FILE_LIST_NUM_COLUMNS ; i++ )
@@ -1510,38 +1515,17 @@ cleanup_file_format (FileFormatData *data)
 }
 
 
-/******************************************************************************
-*
-*   Function: gnome_cmd_file_list_add_file
-*
-*   Purpose:  Add a file to the list
-*
-*   Params:   @fl: The FileList to add the file to
-*             @finfo: The file to add
-*             @in_row: The row to add the file at. Set to -1 to append the file at the end.
-*
-*   Returns: 
-*
-*   Statuses: 
-*
-******************************************************************************/
-void
-gnome_cmd_file_list_add_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo, gint in_row)
+static void
+add_file_to_clist (GnomeCmdFileList *fl, GnomeCmdFile *finfo, gint in_row)
 {
 	gint row;
-	FileFormatData data;
 	GtkCList *clist;
+	FileFormatData data;
 
 	clist = GTK_CLIST (fl);
 
 	format_file_for_display (finfo, &data, FALSE);
 	
-
-	/* Add the file to the list
-	 *
-	 */
-	gnome_cmd_file_ref (finfo);
-	fl->priv->shown_files = g_list_append (fl->priv->shown_files, finfo);	
 	if (in_row == -1)
 		row = gtk_clist_append (clist, data.text);	
 	else
@@ -1575,6 +1559,38 @@ gnome_cmd_file_list_add_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo, gint in
 							  gnome_cmd_file_get_type_pixmap (finfo),
 							  gnome_cmd_file_get_type_mask (finfo));
 	}
+
+	/* If we have been waiting for this file to show up, focus it */
+	if (fl->priv->focus_later &&
+		strcmp (gnome_cmd_file_get_name (finfo), fl->priv->focus_later) == 0)
+		focus_file_at_row (fl, row);
+}
+
+
+/******************************************************************************
+*
+*   Function: gnome_cmd_file_list_add_file
+*
+*   Purpose:  Add a file to the list
+*
+*   Params:   @fl: The FileList to add the file to
+*             @finfo: The file to add
+*             @in_row: The row to add the file at. Set to -1 to append the file at the end.
+*
+*   Returns: 
+*
+*   Statuses: 
+*
+******************************************************************************/
+void
+gnome_cmd_file_list_add_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo, gint row)
+{
+	/* Add the file to the list
+	 *
+	 */
+	gnome_cmd_file_collection_add (fl->priv->shown_files, finfo);
+
+	add_file_to_clist (fl, finfo, row);
 }
 
 
@@ -1699,14 +1715,28 @@ gnome_cmd_file_list_remove_file (GnomeCmdFileList *fl, GnomeCmdFile *finfo)
 	if (row >= 0) {
 		gtk_clist_remove (GTK_CLIST (fl), row);
 		fl->priv->selected_files = g_list_remove (fl->priv->selected_files, finfo);
-		fl->priv->shown_files = g_list_remove (fl->priv->shown_files, finfo);
-		gnome_cmd_file_unref (finfo);
+		gnome_cmd_file_collection_remove (fl->priv->shown_files, finfo);
 
 		focus_file_at_row (fl, MIN(row, GTK_CLIST(fl)->focus_row));
 	}
 }
 
 
+void
+gnome_cmd_file_list_remove_file_by_uri (GnomeCmdFileList *fl, const gchar *uri_str)
+{
+	GnomeCmdFile *file;
+	
+	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
+	g_return_if_fail (uri_str != NULL);
+
+	file = gnome_cmd_file_collection_lookup (fl->priv->shown_files, uri_str);
+	g_return_if_fail (GNOME_CMD_IS_FILE (file));
+
+	gnome_cmd_file_list_remove_file (fl, file);
+}
+
+
 void
 gnome_cmd_file_list_remove_files (GnomeCmdFileList *fl, GList *files)
 {
@@ -1724,9 +1754,8 @@ gnome_cmd_file_list_remove_all_files (GnomeCmdFileList *fl)
 	g_return_if_fail (GNOME_CMD_IS_FILE_LIST (fl));
 	
 	gtk_clist_clear (GTK_CLIST (fl));
-	gnome_cmd_file_list_free (fl->priv->shown_files);
+	gnome_cmd_file_collection_clear (fl->priv->shown_files);
 	gnome_cmd_file_list_free (fl->priv->selected_files);
-	fl->priv->shown_files = NULL;
 	fl->priv->selected_files = NULL;
 }
 
@@ -1816,7 +1845,7 @@ gnome_cmd_file_list_get_all_files (GnomeCmdFileList *fl)
 {
 	g_return_val_if_fail (GNOME_CMD_IS_FILE_LIST (fl), NULL);
 	
-	return fl->priv->shown_files;
+	return gnome_cmd_file_collection_get_list (fl->priv->shown_files);
 }
 
 
@@ -1981,6 +2010,12 @@ gnome_cmd_file_list_focus_file (GnomeCmdFileList *fl,
 		}
 		tmp = tmp->next;
 	}
+
+	/* The file was not found, remember the filename in case the file gets
+	   added to the list in the future (after a FAM event etc). */
+	if (fl->priv->focus_later)
+		g_free (fl->priv->focus_later);
+	fl->priv->focus_later = g_strdup (focus_file);
 }
 
 
@@ -2131,22 +2166,21 @@ gnome_cmd_file_list_sort (GnomeCmdFileList *fl)
 	gtk_clist_clear (GTK_CLIST (fl));
 
 	/* Resort the files and readd them to the list */
-	list = g_list_copy (gnome_cmd_file_list_get_all_files (fl));
+	list = gnome_cmd_file_list_get_all_files (fl);
 	list = gnome_vfs_list_sort (list, fl->priv->sort_func, fl);
 	while (list != NULL)
 	{
 		GnomeCmdFile *finfo;
 
 		finfo = GNOME_CMD_FILE (list->data);
-		fl->priv->shown_files = g_list_remove (
-			fl->priv->shown_files, finfo);
-		gnome_cmd_file_list_add_file (fl, finfo, -1);		
+		add_file_to_clist (fl, finfo, -1);
 		list = list->next;
 	}
 	g_list_free (list);
 
-	/* refocus the previously selected file */
-	if (selfile) {
+	/* refocus the previously selected file if this
+	   file-list has the focus */
+	if (selfile && GTK_WIDGET_HAS_FOCUS (fl)) {
 		selrow = get_row_from_file (fl, selfile);
 		gnome_cmd_file_list_select_row (fl, selrow);
 		gtk_clist_moveto (GTK_CLIST (fl), selrow, -1, 1, 0);
@@ -2406,11 +2440,12 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 				break;
 		}
 	}
-	else if (state_is_ctrl_alt (event->state)) {
+	else if (state_is_ctrl_alt (event->state) || state_is_ctrl_alt_shift (event->state)) {
 		if ((event->keyval >= GDK_a && event->keyval <= GDK_z)
-			|| (event->keyval >= GDK_A && event->keyval <= GDK_Z))
+			|| (event->keyval >= GDK_A && event->keyval <= GDK_Z)
+			|| event->keyval == GDK_period)
 			gnome_cmd_file_list_show_quicksearch (fl, (gchar)event->keyval);
-	}	
+	}
 	else if (state_is_shift (event->state)) {
 		switch (event->keyval)
 		{
@@ -2421,6 +2456,10 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 			case GDK_F6:
 				gnome_cmd_file_list_show_rename_dialog (fl);
 				return TRUE;
+
+			case GDK_F10:
+				show_file_popup_with_warp (fl);
+				return TRUE;
 				
 			case GDK_Left:
 			case GDK_Right:
@@ -2483,6 +2522,8 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 				gnome_cmd_file_list_cap_copy (fl);
 				return TRUE;
 
+			case GDK_m:
+			case GDK_M:
 			case GDK_t:
 			case GDK_T:
 				gnome_cmd_file_list_show_advrename_dialog (fl);
@@ -2491,9 +2532,11 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 			case GDK_a:
 			case GDK_A:	
 			case GDK_KP_Add:
+			case GDK_equal:
 				gnome_cmd_file_list_select_all (fl);
 				return TRUE;
 
+			case GDK_minus:
 			case GDK_KP_Subtract:
 				gnome_cmd_file_list_unselect_all (fl);
 				return TRUE;
@@ -2520,10 +2563,13 @@ gnome_cmd_file_list_keypressed (GnomeCmdFileList *fl,
 				return mime_exec_file (gnome_cmd_file_list_get_focused_file (fl));
 				
 			case GDK_KP_Add:
+			case GDK_plus:
+			case GDK_equal:
 				show_selpat_dialog (fl, TRUE);
 				return TRUE;
 
 			case GDK_KP_Subtract:
+			case GDK_minus:
 				show_selpat_dialog (fl, FALSE);
 				return TRUE;
 
@@ -2655,3 +2701,10 @@ gnome_vfs_list_sort (GList *list,
 		 compare_func,
 		 data);
 }
+
+
+GList *
+gnome_cmd_file_list_sort_selection (GList *list, GnomeCmdFileList *fl)
+{
+	return gnome_vfs_list_sort(list, (GnomeVFSListCompareFunc)fl->priv->sort_func, fl);
+}
