@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2004 Marcus Bjurman
+    Copyright (C) 2001-2006 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -196,6 +196,9 @@ gnome_cmd_file_list_compare_directories (void);
 void
 gnome_cmd_file_list_sort (GnomeCmdFileList *fl);
 
+GList *
+gnome_cmd_file_list_sort_selection (GList *list, GnomeCmdFileList *fl);
+
 GnomeCmdFile *
 gnome_cmd_file_list_get_file_at_row (GnomeCmdFileList *fl, gint row);
 
