@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2003 Marcus Bjurman
+    Copyright (C) 2001-2006 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "gnome-cmd-file-props-dialog.h"
 #include "gnome-cmd-main-win.h"
 #include "gnome-cmd-xfer.h"
+#include "gnome-cmd-internal-viewer.h"
 
 #define MAX_TYPE_LENGTH 2
 #define MAX_NAME_LENGTH 128
@@ -303,14 +304,11 @@ gnome_cmd_file_rename (GnomeCmdFile *finfo, const gchar *new_name)
 		GNOME_VFS_SET_FILE_INFO_NAME);
 	gnome_vfs_uri_unref (uri);
 
-	if (result == GNOME_VFS_OK) {
-		if (has_parent_dir (finfo)) {
-			GnomeCmdDir *dir = get_parent_dir (finfo);
-			if (!gnome_cmd_dir_uses_fam (dir)) {
-				gnome_cmd_file_update_info (finfo, new_info);
-				gnome_cmd_dir_file_renamed (dir, finfo);
-			}
-		}
+	if (result == GNOME_VFS_OK && has_parent_dir (finfo)) {
+		gnome_cmd_file_update_info (finfo, new_info);
+		gnome_cmd_dir_file_renamed (get_parent_dir (finfo), finfo);
+		if (GNOME_CMD_IS_DIR (finfo))
+			gnome_cmd_dir_update_path (GNOME_CMD_DIR (finfo));
 	}
 
 	return result;
@@ -369,10 +367,10 @@ gnome_cmd_file_get_path (GnomeCmdFile *finfo)
 gchar *
 gnome_cmd_file_get_real_path (GnomeCmdFile *finfo)
 {
-	gchar *uri_str = gnome_cmd_file_get_uri_str (finfo);
-	gchar *path = gnome_vfs_get_local_path_from_uri (uri_str);
+	GnomeVFSURI *uri = gnome_cmd_file_get_uri (finfo);
+	gchar *path = gnome_vfs_unescape_string(gnome_vfs_uri_get_path (uri),NULL);
 
-	g_free (uri_str);
+	gnome_vfs_uri_unref (uri);
 	
 	return path;
 
@@ -713,15 +711,22 @@ gnome_cmd_file_show_properties (GnomeCmdFile *finfo)
 static void
 do_view_file (const gchar *path)
 {
-	gchar *command;
 	gchar *arg;
+	gchar *command;
 
-	arg = g_shell_quote (path);
-	command = g_strdup_printf (gnome_cmd_data_get_viewer (), arg);
-	run_command (command, FALSE);
-
-	g_free (arg);
-	g_free (command);
+	if (gnome_cmd_data_get_use_internal_viewer()) {
+		arg = gnome_vfs_unescape_string (path, NULL);
+		do_internal_file_view(arg);
+		g_free(arg);
+	}
+	else {
+		arg = g_shell_quote (path);
+		command = g_strdup_printf (gnome_cmd_data_get_viewer (), arg);
+		run_command (command, FALSE);
+		g_free (arg);
+		g_free (command);
+	}
+		
 }
 
 
@@ -822,16 +827,6 @@ gnome_cmd_file_update_info (GnomeCmdFile *finfo, GnomeVFSFileInfo *info)
 }
 
 
-void
-gnome_cmd_file_is_deleted (GnomeCmdFile *finfo)
-{
-	g_return_if_fail (GNOME_CMD_IS_FILE (finfo));
-
-	if (has_parent_dir (finfo)
-		&& !gnome_cmd_dir_uses_fam (get_parent_dir (finfo)))
-		gnome_cmd_dir_file_deleted (get_parent_dir (finfo), finfo->info->name);
-}
-
 gboolean
 gnome_cmd_file_is_local (GnomeCmdFile *finfo)
 {
@@ -872,6 +867,19 @@ gnome_cmd_file_is_executable (GnomeCmdFile *finfo)
 }
 
 
+void
+gnome_cmd_file_is_deleted (GnomeCmdFile *finfo)
+{
+	g_return_if_fail (GNOME_CMD_IS_FILE (finfo));
+
+	if (has_parent_dir (finfo)) {
+		gchar *uri_str = gnome_cmd_file_get_uri_str (finfo);
+		gnome_cmd_dir_file_deleted (get_parent_dir (finfo), uri_str);
+		g_free (uri_str);
+	}
+}
+
+
 void
 gnome_cmd_file_execute (GnomeCmdFile *finfo)
 {
