@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2003 Marcus Bjurman
+    Copyright (C) 2001-2006 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,27 +17,27 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */ 
 #include <config.h>
-#include <pthread.h>
 #include <sys/wait.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-cmdline.h"
-#include "gnome-cmd-file-selector-types.h"
-#include "gnome-cmd-file-selector-funcs.h"
+#include "gnome-cmd-file-selector.h"
 #include "useractions.h"
-#include "gnome-cmd-main-win-funcs.h"
+#include "plugin_manager.h"
+#include "gnome-cmd-main-win.h"
 #include "gnome-cmd-main-menu.h"
 #include "gnome-cmd-data.h"
 #include "gnome-cmd-style.h"
 #include "gnome-cmd-combo.h"
-#include "gnome-cmd-dir-funcs.h"
-#include "gnome-cmd-connection-funcs.h"
+#include "gnome-cmd-dir.h"
+#include "gnome-cmd-plain-path.h"
+#include "gnome-cmd-con.h"
 #include "gnome-cmd-bookmark-dialog.h"
-#include "dir_pool.h"
 #include "utils.h"
 
 #include "../pixmaps/ftp_connect.xpm"
+#include "../pixmaps/copy_file_names.xpm"
 
 
 enum {
@@ -54,19 +54,15 @@ typedef enum {
 struct _GnomeCmdMainWinPrivate
 {
 	FileSelectorID current_fs;
+	GnomeCmdState state;
 	
 	GtkWidget *main_win;
 	GtkWidget *vbox;
-	GtkWidget *hpaned;
+	GtkWidget *paned;
 	GtkWidget *file_selector[2];
-	GtkWidget *cmdline;
+	GtkWidget *focused_widget;
 	GtkAccelGroup *accel_group;
 
-	GtkWidget *back_btn;
-	GtkWidget *forward_btn;
-	
-	GtkWidget *con_drop_btn;
-
 	GtkWidget *view_btn;
 	GtkWidget *edit_btn;
 	GtkWidget *copy_btn;
@@ -79,15 +75,25 @@ struct _GnomeCmdMainWinPrivate
 	GtkWidget *menubar;
 	GtkWidget *toolbar;
 	GtkWidget *toolbar_sep;
+	GtkWidget *cmdline;
+	GtkWidget *cmdline_sep;
 	GtkWidget *buttonbar;
 	GtkWidget *buttonbar_sep;
+
+	GtkWidget *tb_back_btn;
+	GtkWidget *tb_fwd_btn;
+	GtkWidget *tb_con_drop_btn;
+	GtkWidget *tb_cap_cut_btn;
+	GtkWidget *tb_cap_copy_btn;
+	GtkWidget *tb_cap_paste_btn;
 };
 
 static GnomeAppClass *parent_class = NULL;
 
 static guint main_win_signals[LAST_SIGNAL] = { 0 };
 static GtkTooltips *toolbar_tooltips = NULL;
-extern gchar *start_dir; //main.c
+extern gchar *start_dir_left; //main.c
+extern gchar *start_dir_right; //main.c
 
 static void
 gnome_cmd_main_win_real_switch_fs (GnomeCmdMainWin *mw, GnomeCmdFileSelector *fs);
@@ -109,12 +115,8 @@ add_buttonbar_button (char *label,
 							  button,
 							  (GtkDestroyNotify) gtk_widget_unref);
 	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
-	gtk_widget_modify_style (GTK_BIN (button)->child, main_rc_style);
 
 	gtk_box_pack_start (GTK_BOX (mw->priv->buttonbar), button, TRUE, TRUE, 0);
-	gtk_widget_add_accelerator (button, "clicked",
-								accel_group, accel_signal_id,
-								0, GTK_ACCEL_VISIBLE);
 	return button;
 }
 
@@ -167,6 +169,7 @@ create_toolbar (GnomeCmdMainWin *mw, GnomeUIInfo *uiinfo)
 			w = create_separator (TRUE);
 		}
 		else {
+			
 			GtkWidget *pixmap;
 			
 			w = create_styled_button (NULL);
@@ -176,7 +179,10 @@ create_toolbar (GnomeCmdMainWin *mw, GnomeUIInfo *uiinfo)
 				toolbar_tooltips, w, uiinfo[i].hint, NULL);
 			GTK_WIDGET_UNSET_FLAGS (w, GTK_CAN_FOCUS);			
 			
-			pixmap = create_ui_pixmap (GTK_WIDGET (mw), uiinfo[i].pixmap_type, uiinfo[i].pixmap_info);
+			pixmap = create_ui_pixmap (
+				GTK_WIDGET (mw),
+				uiinfo[i].pixmap_type, uiinfo[i].pixmap_info,
+				GTK_ICON_SIZE_LARGE_TOOLBAR);
 			if (pixmap) {
 				gtk_widget_ref (pixmap);
 				gtk_widget_show (pixmap);			
@@ -185,9 +191,12 @@ create_toolbar (GnomeCmdMainWin *mw, GnomeUIInfo *uiinfo)
 			
 			switch (i)
 			{
-			  case  1:  mw->priv->back_btn = w;  break;
-			  case  2:  mw->priv->forward_btn = w;  break;
-			  case 11:  mw->priv->con_drop_btn = w;  break;
+				case  1: mw->priv->tb_back_btn = w;  break;
+				case  2: mw->priv->tb_fwd_btn = w;  break;
+				case  5: mw->priv->tb_cap_cut_btn = w;  break;
+				case  6: mw->priv->tb_cap_copy_btn = w;  break;
+				case  7: mw->priv->tb_cap_paste_btn = w;  break;
+				case 11: mw->priv->tb_con_drop_btn = w;  break;
 			}
 		}
 
@@ -196,13 +205,14 @@ create_toolbar (GnomeCmdMainWin *mw, GnomeUIInfo *uiinfo)
 	}
 
 	mw->priv->toolbar_sep = create_separator (FALSE);
+	gtk_widget_set_sensitive (mw->priv->tb_cap_paste_btn, FALSE);
 }
 
 
 static void
 slide_set_100_0 (GtkMenu *menu, gpointer user_data)
 {
-	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
+	gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
 							GTK_WIDGET (main_win)->allocation.width);	
 }
 
@@ -210,7 +220,7 @@ slide_set_100_0 (GtkMenu *menu, gpointer user_data)
 static void
 slide_set_80_20 (GtkMenu *menu, gpointer user_data)
 {
-	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
+	gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
 							(int)(GTK_WIDGET (main_win)->allocation.width*0.8f));	
 }
 
@@ -218,7 +228,7 @@ slide_set_80_20 (GtkMenu *menu, gpointer user_data)
 static void
 slide_set_60_40 (GtkMenu *menu, gpointer user_data)
 {
-	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
+	gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
 							(int)(GTK_WIDGET (main_win)->allocation.width*0.6f));	
 }
 
@@ -226,7 +236,7 @@ slide_set_60_40 (GtkMenu *menu, gpointer user_data)
 static void
 slide_set_50_50 (GtkMenu *menu, gpointer user_data)
 {
-	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
+	gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
 							(int)(GTK_WIDGET (main_win)->allocation.width*0.5f));	
 }
 
@@ -234,7 +244,7 @@ slide_set_50_50 (GtkMenu *menu, gpointer user_data)
 static void
 slide_set_40_60 (GtkMenu *menu, gpointer user_data)
 {
-	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
+	gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
 							(int)(GTK_WIDGET (main_win)->allocation.width*0.4f));	
 }
 
@@ -242,7 +252,7 @@ slide_set_40_60 (GtkMenu *menu, gpointer user_data)
 static void
 slide_set_20_80 (GtkMenu *menu, gpointer user_data)
 {
-	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned),
+	gtk_paned_set_position (GTK_PANED (main_win->priv->paned),
 							(int)(GTK_WIDGET (main_win)->allocation.width*0.2f));	
 }
 
@@ -250,7 +260,7 @@ slide_set_20_80 (GtkMenu *menu, gpointer user_data)
 static void
 slide_set_0_100 (GtkMenu *menu, gpointer user_data)
 {
-	gtk_paned_set_position (GTK_PANED (main_win->priv->hpaned), 0);	
+	gtk_paned_set_position (GTK_PANED (main_win->priv->paned), 0);	
 }
 
 
@@ -351,10 +361,10 @@ on_help_clicked                        (GtkButton       *button,
 
 
 static void
-on_refresh_clicked                        (GtkButton       *button,
-									   GnomeCmdMainWin *mw)
+on_rename_clicked                      (GtkButton       *button,
+										GnomeCmdMainWin *mw)
 {
-	view_refresh (NULL, mw);
+	file_rename (NULL, mw);
 }
 
 
@@ -418,7 +428,7 @@ static void
 on_quit_clicked                        (GtkButton       *button,
                                         GnomeCmdMainWin *mw)
 {
-    gtk_main_quit ();
+	file_exit (NULL, mw);
 }
 
 
@@ -480,48 +490,53 @@ create_buttonbar (GnomeCmdMainWin *mw)
 	Misc widgets callbacks
 *****************************************************************************/
 
-static void
+static gboolean
 on_slide_button_press (GtkWidget *widget, GdkEventButton *event, GnomeCmdMainWin *mw)
 {
 	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
-		GtkPaned *paned = GTK_PANED (mw->priv->hpaned);
+		GtkPaned *paned = GTK_PANED (mw->priv->paned);
 
 		/* Check that the handle was clicked and not one of the children */
-		if (paned->handle == event->window)
+		if (paned->handle == event->window) {
 			gtk_menu_popup (GTK_MENU(create_slide_popup ()), NULL, NULL, NULL, NULL, 
 							event->button, event->time);
+			return TRUE;
+		}
 	}
+
+	return FALSE;
 }
 
 static void
 on_main_win_realize                    (GtkWidget       *widget,
                                         GnomeCmdMainWin *mw)
 {
+	gint w;
+	
+	gdk_window_get_geometry (widget->window, NULL, NULL, &w, NULL, NULL);
+	gtk_paned_set_position (GTK_PANED (mw->priv->paned), w/2 - 5);
+	
 	gnome_cmd_file_selector_set_active (
 		gnome_cmd_main_win_get_left_fs (mw), TRUE);
-	
-	gnome_cmd_cmdline_set_dir (
-		GNOME_CMD_CMDLINE (mw->priv->cmdline),
-		gnome_cmd_dir_get_path (
-			gnome_cmd_file_selector_get_directory (
-				gnome_cmd_main_win_get_left_fs (mw))));
-	
+	gnome_cmd_file_selector_set_active (
+		gnome_cmd_main_win_get_right_fs (mw), FALSE);
+/*
+	if (gnome_cmd_data_get_cmdline_visibility ()) {
+		gchar *dpath = gnome_cmd_file_get_path (
+				GNOME_CMD_FILE (gnome_cmd_file_selector_get_directory (
+									gnome_cmd_main_win_get_left_fs (mw))));
+		gnome_cmd_cmdline_set_dir (
+			GNOME_CMD_CMDLINE (mw->priv->cmdline), dpath);
+		g_free (dpath);
+	}
+*/		
 	gdk_window_set_icon (GTK_WIDGET (main_win)->window, NULL,
 						 IMAGE_get_pixmap (PIXMAP_LOGO),
 						 IMAGE_get_mask (PIXMAP_LOGO));
 }
 
 
-static void
-on_main_win_size_allocate              (GtkWidget       *widget,
-                                        GtkAllocation   *allocation,
-                                        GnomeCmdMainWin *mw)
-{
-	gtk_paned_set_position (GTK_PANED (mw->priv->hpaned), allocation->width/2 - 5);
-}
-
-
-static void
+static gboolean
 on_left_fs_select                      (GtkCList *list,
 										GdkEventButton *event,
                                         GnomeCmdMainWin *mw)
@@ -532,11 +547,13 @@ on_left_fs_select                      (GtkCList *list,
 			GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]), TRUE);
 		gnome_cmd_file_selector_set_active (
 			GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]), FALSE);
+
+		return FALSE;
 }
 
 
 
-static void
+static gboolean
 on_right_fs_select                     (GtkCList *list,
 										GdkEventButton *event,
                                         GnomeCmdMainWin *mw)
@@ -547,6 +564,8 @@ on_right_fs_select                     (GtkCList *list,
 			GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]), TRUE);
 		gnome_cmd_file_selector_set_active (
 			GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]), FALSE);
+
+		return FALSE;
 }
 
 
@@ -575,7 +594,7 @@ on_size_allocate                  (GtkWidget *widget,
 								   GtkAllocation *allocation,
 								   gpointer user_data)
 {
-	gnome_cmd_data_set_position (allocation->width, allocation->height);
+	gnome_cmd_data_set_main_win_size (allocation->width, allocation->height);
 }
 
 
@@ -588,9 +607,9 @@ update_browse_buttons             (GnomeCmdMainWin *mw,
 	
 	if (fs == gnome_cmd_main_win_get_active_fs (mw)) {
 		if (gnome_cmd_data_get_toolbar_visibility ()) {
-			gtk_widget_set_sensitive (mw->priv->back_btn,
+			gtk_widget_set_sensitive (mw->priv->tb_back_btn,
 									  gnome_cmd_file_selector_can_back (fs));
-			gtk_widget_set_sensitive (mw->priv->forward_btn,
+			gtk_widget_set_sensitive (mw->priv->tb_fwd_btn,
 									  gnome_cmd_file_selector_can_forward (fs));
 		}
 
@@ -605,33 +624,29 @@ update_drop_con_button            (GnomeCmdMainWin *mw,
 {
 	GtkWidget *btn;
 	GnomeCmdPixmap *pm;
-	GnomeCmdConnection *con;
+	GnomeCmdCon *con;
 	static GtkWidget *prev_pixmap = NULL;
 
 	g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
 	g_return_if_fail (GNOME_CMD_IS_FILE_SELECTOR (fs));
 
 	con = gnome_cmd_file_selector_get_connection (fs);
-	g_return_if_fail (con != NULL);
-
 	if (!gnome_cmd_data_get_toolbar_visibility ()
-		|| (gnome_cmd_data_get_skip_mounting ()
-			&& con->type == CON_TYPE_DEV))
+		|| (gnome_cmd_data_get_skip_mounting () && GNOME_CMD_IS_CON_DEVICE (con)))
 		return;
 	
-	btn = mw->priv->con_drop_btn;
+	btn = mw->priv->tb_con_drop_btn;
+	if (!con)
+		return;
 	g_return_if_fail (GTK_IS_BUTTON (btn));
-	
+
 	if (prev_pixmap) {
-		gtk_object_destroy (GTK_OBJECT (prev_pixmap));
+		gtk_widget_destroy (prev_pixmap);
 		prev_pixmap = NULL;
 	}
 	
-	if (con->type == CON_TYPE_DEV) {
-		pm = gnome_cmd_device_get_umount_pixmap (con->device);
-	}
-	else if (con->type == CON_TYPE_FTP) {
-		pm = IMAGE_get_gnome_cmd_pixmap (PIXMAP_FTP_DISCONNECT);		
+	if (gnome_cmd_con_is_closeable (con)) {
+		pm = gnome_cmd_con_get_close_pixmap (con);
 	}
 	else {
 		gtk_widget_set_sensitive (btn, FALSE);
@@ -639,7 +654,7 @@ update_drop_con_button            (GnomeCmdMainWin *mw,
 	}
 
 	g_return_if_fail (toolbar_tooltips != NULL);
-	gtk_tooltips_set_tip (toolbar_tooltips, btn, con->con_drop_tooltip, NULL);
+	gtk_tooltips_set_tip (toolbar_tooltips, btn, gnome_cmd_con_get_close_tooltip (con), NULL);
 	gtk_widget_set_sensitive (btn, TRUE);
 
 	if (pm) {
@@ -652,7 +667,7 @@ update_drop_con_button            (GnomeCmdMainWin *mw,
 		}
 	}
 	else {
-		GtkWidget *label = gtk_label_new (con->con_drop_text);
+		GtkWidget *label = gtk_label_new (gnome_cmd_con_get_close_text (con));
 		if (label) {
 			gtk_widget_ref (label);
 			gtk_widget_show (label);
@@ -673,6 +688,27 @@ on_fs_dir_change                  (GnomeCmdFileSelector *fs,
 }
 
 
+static void
+restore_size_and_pos (GnomeCmdMainWin *mw)
+{
+	gint w, h, x, y;
+	
+	gnome_cmd_data_get_main_win_size (&w, &h);
+	gtk_widget_set_usize (GTK_WIDGET (main_win), w, h);
+
+	gnome_cmd_data_get_main_win_pos (&x, &y);
+	if (x >= 0 && y >= 0)
+		gtk_window_move (GTK_WINDOW (mw), x, y);
+}
+
+
+static void
+on_delete_event (GnomeCmdMainWin *mw, GdkEvent *event, gpointer user_data)
+{
+	file_exit (NULL, mw);
+}
+
+
 /*******************************
  * Gtk class implementation
  *******************************/
@@ -680,6 +716,24 @@ on_fs_dir_change                  (GnomeCmdFileSelector *fs,
 static void
 destroy (GtkObject *object)
 {
+	GnomeCmdDir *dir;
+	GnomeCmdCon *con_home;
+
+	con_home = gnome_cmd_con_list_get_home (
+		gnome_cmd_data_get_con_list ());
+	
+	dir = gnome_cmd_file_selector_get_directory (
+		gnome_cmd_main_win_get_left_fs (main_win));
+	if (con_home == gnome_cmd_dir_get_connection (dir))	
+		gnome_cmd_data_set_start_dir (
+			0, gnome_cmd_file_get_path (GNOME_CMD_FILE (dir)));
+		
+	dir = gnome_cmd_file_selector_get_directory (
+		gnome_cmd_main_win_get_right_fs (main_win));
+	if (con_home == gnome_cmd_dir_get_connection (dir))
+		gnome_cmd_data_set_start_dir (
+			1, gnome_cmd_file_get_path (GNOME_CMD_FILE (dir)));
+
 	gtk_main_quit ();
 }
 
@@ -705,13 +759,12 @@ class_init (GnomeCmdMainWinClass *class)
 	main_win_signals[SWITCH_FS] =
 		gtk_signal_new ("switch_fs",
 			GTK_RUN_LAST,
-		    object_class->type,
+		    G_OBJECT_CLASS_TYPE (object_class),
 		    GTK_SIGNAL_OFFSET (GnomeCmdMainWinClass, switch_fs),
 		    gtk_marshal_NONE__POINTER,
 		    GTK_TYPE_NONE,
 			1, GTK_TYPE_POINTER);
   
-	gtk_object_class_add_signals (object_class, main_win_signals, LAST_SIGNAL);
 	object_class->destroy = destroy;
 	widget_class->map = map;
 	class->switch_fs = gnome_cmd_main_win_real_switch_fs;
@@ -721,8 +774,6 @@ class_init (GnomeCmdMainWinClass *class)
 static void
 init (GnomeCmdMainWin *mw)
 {
-	gint main_win_width, main_win_height;
-	
 	/* It is very important that this global variable gets assigned here so that
 	 * child widgets to this window can use that variable when initializing
 	 */
@@ -731,18 +782,20 @@ init (GnomeCmdMainWin *mw)
 	mw->priv = g_new (GnomeCmdMainWinPrivate, 1);
 	mw->priv->current_fs = LEFT;
 	mw->priv->accel_group = gtk_accel_group_new ();
-	mw->priv->buttonbar = NULL;
-	mw->priv->buttonbar_sep = NULL;
 	mw->priv->toolbar = NULL;
 	mw->priv->toolbar_sep = NULL;
+	mw->priv->focused_widget = NULL;
+	mw->priv->cmdline = NULL;
+	mw->priv->cmdline_sep = NULL;
+	mw->priv->buttonbar = NULL;
+	mw->priv->buttonbar_sep = NULL;
+	mw->priv->file_selector[0] = NULL;
+	mw->priv->file_selector[1] = NULL;
 
-	gnome_cmd_data_get_position (&main_win_width, &main_win_height);
-
+	gnome_app_construct (GNOME_APP (main_win), "gnome-commander", "GNOME Commander");
 	gtk_object_set_data (GTK_OBJECT (main_win), "main_win", main_win);
-	gtk_window_set_title (GTK_WINDOW (main_win), "GNOME Commander");
-	gtk_window_set_position (GTK_WINDOW (main_win), GTK_WIN_POS_CENTER);
+	restore_size_and_pos (mw);
 	gtk_window_set_policy (GTK_WINDOW (main_win), TRUE, TRUE, FALSE);
-	gtk_widget_set_usize (GTK_WIDGET (main_win), main_win_width, main_win_height);
 
 	mw->priv->vbox = gtk_vbox_new (FALSE, 0);
 	gtk_widget_ref (mw->priv->vbox);
@@ -761,16 +814,16 @@ init (GnomeCmdMainWin *mw)
 	
 	gnome_app_set_contents (GNOME_APP (main_win), mw->priv->vbox);
 
-	mw->priv->hpaned = gtk_hpaned_new ();
-	gtk_widget_ref (mw->priv->hpaned);
-	gtk_object_set_data_full (GTK_OBJECT (main_win), "paned", mw->priv->hpaned,
+	if (gnome_cmd_data_get_list_orientation ())
+		mw->priv->paned = gtk_vpaned_new ();
+	else
+		mw->priv->paned = gtk_hpaned_new ();
+	gtk_widget_ref (mw->priv->paned);
+	gtk_object_set_data_full (GTK_OBJECT (main_win), "paned", mw->priv->paned,
 							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (mw->priv->hpaned);
-	gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->hpaned,
+	gtk_widget_show (mw->priv->paned);
+	gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->paned,
 						TRUE, TRUE, 0);
-	gtk_paned_set_handle_size (GTK_PANED (mw->priv->hpaned), 8);
-	gtk_paned_set_gutter_size (GTK_PANED (mw->priv->hpaned), 10);
-
 	create_buttonbar (mw);
 
 	mw->priv->file_selector[LEFT] = gnome_cmd_file_selector_new ();
@@ -779,7 +832,7 @@ init (GnomeCmdMainWin *mw)
 							  mw->priv->file_selector[LEFT],
 							  (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (mw->priv->file_selector[LEFT]);
-	gtk_paned_pack1 (GTK_PANED (mw->priv->hpaned), mw->priv->file_selector[LEFT],
+	gtk_paned_pack1 (GTK_PANED (mw->priv->paned), mw->priv->file_selector[LEFT],
 					 TRUE, TRUE);
 
 	mw->priv->file_selector[RIGHT] = gnome_cmd_file_selector_new ();
@@ -788,46 +841,33 @@ init (GnomeCmdMainWin *mw)
 							  mw->priv->file_selector[RIGHT],
 							  (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (mw->priv->file_selector[RIGHT]);
-	gtk_paned_pack2 (GTK_PANED (mw->priv->hpaned), mw->priv->file_selector[RIGHT],
+	gtk_paned_pack2 (GTK_PANED (mw->priv->paned), mw->priv->file_selector[RIGHT],
 					 TRUE, TRUE);
 
-	gnome_cmd_connection_create_local ();
-	gnome_cmd_connection_create_devices ();
+	/*
+	if (start_dir_left) 
+		gnome_cmd_file_selector_goto_directory (
+			gnome_cmd_main_win_get_left_fs (mw), start_dir_left);
+	else
+		gnome_cmd_file_selector_goto_directory (
+			gnome_cmd_main_win_get_left_fs (mw), 
+			gnome_cmd_data_get_start_dir (0));
 	
-	gnome_cmd_file_selector_update_connections (gnome_cmd_main_win_get_left_fs (mw));
-	gnome_cmd_file_selector_update_connections (gnome_cmd_main_win_get_right_fs (mw));
+	if (start_dir_right) 
+		gnome_cmd_file_selector_goto_directory (
+			gnome_cmd_main_win_get_right_fs (mw), start_dir_right);
+	else
+		gnome_cmd_file_selector_goto_directory (
+			gnome_cmd_main_win_get_right_fs (mw), 
+			gnome_cmd_data_get_start_dir (1));
+	*/
 
-	gnome_cmd_file_selector_set_connection (gnome_cmd_main_win_get_left_fs (mw),
-											gnome_cmd_connection_get_home (), NULL);
-	gnome_cmd_file_selector_set_connection (gnome_cmd_main_win_get_right_fs (mw),
-											gnome_cmd_connection_get_home (), NULL);
-
-	if (start_dir) {
-		gchar *s = g_strdup_printf ("file://%s", start_dir);
-		g_print ("start_dir: %s\n", s);
-		gnome_cmd_file_selector_set_directory (
-			gnome_cmd_main_win_get_left_fs (mw),
-			gnome_cmd_dir_new_from_uri_str (s, TRUE));
-	}
-
-	mw->priv->cmdline = gnome_cmd_cmdline_new ();
-	gtk_widget_ref (mw->priv->cmdline);
-	gtk_object_set_data_full (GTK_OBJECT (main_win), "cmdline", mw->priv->cmdline,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (mw->priv->cmdline);
-	gtk_box_pack_start (GTK_BOX (mw->priv->vbox), create_separator (FALSE), FALSE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->cmdline, FALSE, TRUE, 1);
-
-	gnome_cmd_main_win_set_toolbar_visibility (
-		main_win, gnome_cmd_data_get_toolbar_visibility ());
-	
-	gnome_cmd_main_win_set_buttonbar_visibility (
-		main_win, gnome_cmd_data_get_buttonbar_visibility ());
+	gnome_cmd_main_win_update_toolbar_visibility (main_win);
+	gnome_cmd_main_win_update_cmdline_visibility (main_win);
+	gnome_cmd_main_win_update_buttonbar_visibility (main_win);
 	
 	gtk_signal_connect (GTK_OBJECT (main_win), "realize",
 						GTK_SIGNAL_FUNC (on_main_win_realize), mw);
-	gtk_signal_connect (GTK_OBJECT (main_win), "size_allocate",
-						GTK_SIGNAL_FUNC (on_main_win_size_allocate), mw);
 	gtk_signal_connect (GTK_OBJECT (mw->priv->file_selector[LEFT]), "changed-dir",
 						GTK_SIGNAL_FUNC (on_fs_dir_change), mw);
 	gtk_signal_connect (GTK_OBJECT (mw->priv->file_selector[RIGHT]), "changed-dir",
@@ -852,10 +892,24 @@ init (GnomeCmdMainWin *mw)
 		GTK_OBJECT (mw), "size-allocate",
 		GTK_SIGNAL_FUNC (on_size_allocate), mw);
 	gtk_signal_connect (
-		GTK_OBJECT (mw->priv->hpaned),
+		GTK_OBJECT (mw), "delete-event",
+		GTK_SIGNAL_FUNC (on_delete_event), mw);
+	gtk_signal_connect (
+		GTK_OBJECT (mw->priv->paned),
 		"button_press_event", GTK_SIGNAL_FUNC (on_slide_button_press), mw);	
 
+	gnome_cmd_file_selector_update_connections (gnome_cmd_main_win_get_left_fs (mw));
+	gnome_cmd_file_selector_update_connections (gnome_cmd_main_win_get_right_fs (mw));
+
+	gnome_cmd_file_selector_set_connection (
+		gnome_cmd_main_win_get_left_fs (mw),
+		gnome_cmd_con_list_get_home (gnome_cmd_data_get_con_list ()), NULL);
+	gnome_cmd_file_selector_set_connection (
+		gnome_cmd_main_win_get_right_fs (mw),
+		gnome_cmd_con_list_get_home (gnome_cmd_data_get_con_list ()), NULL);
+
 	gtk_window_add_accel_group (GTK_WINDOW (main_win), mw->priv->accel_group);
+	gnome_cmd_main_win_focus_file_lists (main_win);
 }
 
 
@@ -899,6 +953,11 @@ gnome_cmd_main_win_new              ()
 GnomeCmdFileSelector*
 gnome_cmd_main_win_get_active_fs         (GnomeCmdMainWin *mw)
 {
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+
+	if (!mw->priv->file_selector[mw->priv->current_fs])
+		return NULL;
+	
 	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[mw->priv->current_fs]);
 }
 
@@ -906,6 +965,11 @@ gnome_cmd_main_win_get_active_fs         (GnomeCmdMainWin *mw)
 GnomeCmdFileSelector*
 gnome_cmd_main_win_get_inactive_fs         (GnomeCmdMainWin *mw)
 {
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+
+	if (!mw->priv->file_selector[!mw->priv->current_fs])
+		return NULL;
+	
 	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[!mw->priv->current_fs]);
 }
 
@@ -913,6 +977,11 @@ gnome_cmd_main_win_get_inactive_fs         (GnomeCmdMainWin *mw)
 GnomeCmdFileSelector*
 gnome_cmd_main_win_get_left_fs         (GnomeCmdMainWin *mw)
 {
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+	
+	if (!mw->priv->file_selector[LEFT])
+		return NULL;
+	
 	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[LEFT]);
 }
 
@@ -920,6 +989,11 @@ gnome_cmd_main_win_get_left_fs         (GnomeCmdMainWin *mw)
 GnomeCmdFileSelector*
 gnome_cmd_main_win_get_right_fs         (GnomeCmdMainWin *mw)
 {
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+	
+	if (!mw->priv->file_selector[RIGHT])
+		return NULL;
+	
 	return GNOME_CMD_FILE_SELECTOR (mw->priv->file_selector[RIGHT]);
 }
 
@@ -930,30 +1004,23 @@ gnome_cmd_main_win_update_style          (GnomeCmdMainWin *mw)
 	g_return_if_fail (mw != NULL);
 	g_return_if_fail (mw->priv != NULL);
 
-	if (mw->priv->buttonbar) {
-		gtk_widget_modify_style (GTK_BIN (mw->priv->view_btn)->child, main_rc_style);
-		gtk_widget_modify_style (GTK_BIN (mw->priv->edit_btn)->child, main_rc_style);
-		gtk_widget_modify_style (GTK_BIN (mw->priv->copy_btn)->child, main_rc_style);
-		gtk_widget_modify_style (GTK_BIN (mw->priv->move_btn)->child, main_rc_style);
-		gtk_widget_modify_style (GTK_BIN (mw->priv->mkdir_btn)->child, main_rc_style);
-		gtk_widget_modify_style (GTK_BIN (mw->priv->delete_btn)->child, main_rc_style);
-		gtk_widget_modify_style (GTK_BIN (mw->priv->find_btn)->child, main_rc_style);
-		gtk_widget_modify_style (GTK_BIN (mw->priv->quit_btn)->child, main_rc_style);
-	}
-
 	IMAGE_clear_mime_cache ();
 
 	gnome_cmd_file_selector_update_style (gnome_cmd_main_win_get_left_fs (mw));
 	gnome_cmd_file_selector_update_style (gnome_cmd_main_win_get_right_fs (mw));
 
-	gnome_cmd_cmdline_update_style (GNOME_CMD_CMDLINE (mw->priv->cmdline));
+	if (gnome_cmd_data_get_cmdline_visibility ())	
+		gnome_cmd_cmdline_update_style (GNOME_CMD_CMDLINE (mw->priv->cmdline));
 }
 
 
 void
 gnome_cmd_main_win_focus_cmdline         (GnomeCmdMainWin *mw)
 {
-	gnome_cmd_cmdline_focus (GNOME_CMD_CMDLINE (mw->priv->cmdline));
+	if (gnome_cmd_data_get_cmdline_visibility ()) {
+		gnome_cmd_cmdline_focus (GNOME_CMD_CMDLINE (mw->priv->cmdline));
+		mw->priv->focused_widget = mw->priv->cmdline;
+	}
 }
 
 
@@ -964,6 +1031,16 @@ gnome_cmd_main_win_focus_file_lists      (GnomeCmdMainWin *mw)
 		gnome_cmd_main_win_get_active_fs (mw), TRUE);
 	gnome_cmd_file_selector_set_active (
 		gnome_cmd_main_win_get_inactive_fs (mw), FALSE);
+	
+	mw->priv->focused_widget = mw->priv->file_selector[mw->priv->current_fs];
+}
+
+
+void
+gnome_cmd_main_win_refocus               (GnomeCmdMainWin *mw)
+{
+	if (mw->priv->focused_widget)
+		gtk_widget_grab_focus (mw->priv->focused_widget);
 }
 
 
@@ -998,8 +1075,9 @@ gnome_cmd_main_win_keypressed            (GnomeCmdMainWin *mw,
 				return TRUE;
 				
 			case GDK_F8:
-				gnome_cmd_cmdline_show_history (
-					GNOME_CMD_CMDLINE (mw->priv->cmdline));				
+				if (gnome_cmd_data_get_cmdline_visibility ())
+					gnome_cmd_cmdline_show_history (
+						GNOME_CMD_CMDLINE (mw->priv->cmdline));				
 				return TRUE;
 		}
 	}
@@ -1013,7 +1091,7 @@ gnome_cmd_main_win_keypressed            (GnomeCmdMainWin *mw,
 				
 			case GDK_F:
 			case GDK_f:
-				ftp_disconnect (NULL, NULL);
+				connections_close_current (NULL, NULL);
 				return TRUE;
 
 		}
@@ -1023,8 +1101,9 @@ gnome_cmd_main_win_keypressed            (GnomeCmdMainWin *mw,
 		switch (event->keyval)
 		{				
 			case GDK_Down:
-				gnome_cmd_cmdline_show_history (
-					GNOME_CMD_CMDLINE (mw->priv->cmdline));				
+				if (gnome_cmd_data_get_cmdline_visibility ())
+					gnome_cmd_cmdline_show_history (
+						GNOME_CMD_CMDLINE (mw->priv->cmdline));				
 				return TRUE;
 				
 			case GDK_u:
@@ -1067,12 +1146,12 @@ gnome_cmd_main_win_keypressed            (GnomeCmdMainWin *mw,
 
 			case GDK_F:
 			case GDK_f:
-				ftp_connect (NULL, NULL);
+				connections_ftp_connect (NULL, NULL);
 				return TRUE;
 
 			case GDK_G:
 			case GDK_g:
-				ftp_quick_connect (NULL, NULL);
+				connections_ftp_quick_connect (NULL, NULL);
 				return TRUE;
 		}
 	}
@@ -1080,10 +1159,30 @@ gnome_cmd_main_win_keypressed            (GnomeCmdMainWin *mw,
 	{
 		switch (event->keyval)
 		{
+			case GDK_P:
+			case GDK_p:
+				plugin_manager_show ();
+				break;
+				
 			case GDK_S:
 			case GDK_s:
-				dir_pool_show_state ();
+				gnome_cmd_dir_pool_show_state (
+					gnome_cmd_con_get_dir_pool (
+						gnome_cmd_file_selector_get_connection (
+							gnome_cmd_main_win_get_active_fs (mw))));
 				break;
+
+			case GDK_f:
+			case GDK_F:
+			{
+				GnomeCmdConFtp *con = GNOME_CMD_CON_FTP (gnome_cmd_con_list_get_all_ftp (
+					gnome_cmd_data_get_con_list ())->data);
+
+				gnome_cmd_file_selector_set_connection (
+					gnome_cmd_main_win_get_active_fs (main_win),
+					GNOME_CMD_CON (con), NULL);
+			}
+			break;
 		}
 	}
 	else if (state_is_blank (event->state))
@@ -1096,12 +1195,12 @@ gnome_cmd_main_win_keypressed            (GnomeCmdMainWin *mw,
 				gnome_cmd_main_win_switch_fs (
 					mw, gnome_cmd_main_win_get_inactive_fs (mw));
 				return TRUE;
-
+				
 			case GDK_F1:
 				on_help_clicked (NULL, mw);
 				return TRUE;
 			case GDK_F2:
-				on_refresh_clicked (NULL, mw);
+				on_rename_clicked (NULL, mw);
 				return TRUE;
 			case GDK_F3:
 				on_view_clicked (NULL, mw);
@@ -1188,8 +1287,7 @@ gnome_cmd_main_win_update_bookmarks (GnomeCmdMainWin *mw)
 
 
 void
-gnome_cmd_main_win_set_toolbar_visibility (GnomeCmdMainWin *mw,
-										   gboolean value)
+gnome_cmd_main_win_update_toolbar_visibility (GnomeCmdMainWin *mw)
 {
 	GnomeUIInfo toolbar_uiinfo[] =
 	{
@@ -1215,6 +1313,13 @@ gnome_cmd_main_win_set_toolbar_visibility (GnomeCmdMainWin *mw,
 			0, 0, NULL
 		},
 		GNOMEUIINFO_SEPARATOR,
+		{
+			GNOME_APP_UI_ITEM, NULL,
+			_("Copy file names"),
+			edit_copy_fnames, NULL, NULL,
+			GNOME_APP_PIXMAP_DATA, copy_file_names_xpm,
+			0, 0, NULL
+		},
 		{
 			GNOME_APP_UI_ITEM, NULL,
 			_("Cut"),
@@ -1247,27 +1352,24 @@ gnome_cmd_main_win_set_toolbar_visibility (GnomeCmdMainWin *mw,
 		{
 			GNOME_APP_UI_ITEM, NULL,
 			_("FTP Connect"),
-			ftp_connect, NULL, NULL,
+			connections_ftp_connect, NULL, NULL,
 			GNOME_APP_PIXMAP_DATA, ftp_connect_xpm,
 			0, 0, NULL
 		},
-		GNOMEUIINFO_SEPARATOR,
 		{
 			GNOME_APP_UI_ITEM, NULL,
-			_("Unmount Device"),
-			file_umount, NULL, NULL,
+			_("Drop connection"),
+			connections_close_current, NULL, NULL,
 			0, 0,
 			0, 0, NULL
 		},
 		GNOMEUIINFO_END
 	};
 
-	gnome_cmd_data_set_toolbar_visibility (value);
+	g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
 
-	if (value) {
+	if (gnome_cmd_data_get_toolbar_visibility ()) {
 		create_toolbar (mw, toolbar_uiinfo);
-		update_drop_con_button (mw, gnome_cmd_main_win_get_active_fs (mw));
-		update_browse_buttons (mw, gnome_cmd_main_win_get_active_fs (mw));
 		gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->toolbar, FALSE, TRUE, 0);
 		gtk_box_reorder_child (GTK_BOX (mw->priv->vbox), mw->priv->toolbar, 2);
 		gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->toolbar_sep, FALSE, TRUE, 0);
@@ -1281,16 +1383,17 @@ gnome_cmd_main_win_set_toolbar_visibility (GnomeCmdMainWin *mw,
 		mw->priv->toolbar = NULL;
 		mw->priv->toolbar_sep = NULL;
 	}
+
+	update_drop_con_button (mw, gnome_cmd_main_win_get_active_fs (mw));
 }
 
 
 void
-gnome_cmd_main_win_set_buttonbar_visibility (GnomeCmdMainWin *mw,
-											 gboolean value)
+gnome_cmd_main_win_update_buttonbar_visibility (GnomeCmdMainWin *mw)
 {
-	gnome_cmd_data_set_buttonbar_visibility (value);
+	g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
 
-	if (value) {
+	if (gnome_cmd_data_get_buttonbar_visibility ()) {
 		create_buttonbar (mw);
 		gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->buttonbar_sep, FALSE, TRUE, 0);
 		gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->buttonbar, FALSE, TRUE, 0);
@@ -1306,4 +1409,147 @@ gnome_cmd_main_win_set_buttonbar_visibility (GnomeCmdMainWin *mw,
 }
 
 
+void
+gnome_cmd_main_win_update_cmdline_visibility (GnomeCmdMainWin *mw)
+{
+	g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
+
+	if (gnome_cmd_data_get_cmdline_visibility ()) {
+		gint pos = 3;
+		mw->priv->cmdline_sep = create_separator (FALSE);
+		mw->priv->cmdline = gnome_cmd_cmdline_new ();
+		gtk_widget_ref (mw->priv->cmdline);
+		gtk_object_set_data_full (GTK_OBJECT (main_win), "cmdline", mw->priv->cmdline,
+								  (GtkDestroyNotify) gtk_widget_unref);
+		gtk_widget_show (mw->priv->cmdline);
+		if (gnome_cmd_data_get_toolbar_visibility ())
+			pos += 2;
+		gtk_box_pack_start (
+			GTK_BOX (mw->priv->vbox), mw->priv->cmdline_sep, FALSE, TRUE, 0);
+		gtk_box_pack_start (
+			GTK_BOX (mw->priv->vbox), mw->priv->cmdline, FALSE, TRUE, 1);
+		gtk_box_reorder_child (GTK_BOX (mw->priv->vbox), mw->priv->cmdline_sep, pos);
+		gtk_box_reorder_child (GTK_BOX (mw->priv->vbox), mw->priv->cmdline, pos+1);
+	}
+	else {
+		if (mw->priv->cmdline)
+			gtk_widget_destroy (mw->priv->cmdline);
+		if (mw->priv->cmdline_sep)
+			gtk_widget_destroy (mw->priv->cmdline_sep);
+		mw->priv->cmdline = NULL;
+		mw->priv->cmdline_sep = NULL;
+	}
+}
+
+
+void
+gnome_cmd_main_win_update_connections (GnomeCmdMainWin *mw)
+{
+	g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
+
+	gnome_cmd_file_selector_update_connections (
+		gnome_cmd_main_win_get_left_fs (mw));
+	gnome_cmd_file_selector_update_connections (
+		gnome_cmd_main_win_get_right_fs (mw));		
+	gnome_cmd_main_menu_update_connections (
+		GNOME_CMD_MAIN_MENU (mw->priv->menubar));
+}
+
+
+void
+gnome_cmd_main_win_update_list_orientation (GnomeCmdMainWin *mw)
+{
+	gint pos = 2;
+	
+	g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
+	
+	gtk_widget_ref (mw->priv->file_selector[0]);
+	gtk_widget_ref (mw->priv->file_selector[1]);
+	gtk_container_remove (GTK_CONTAINER (mw->priv->paned),
+						  mw->priv->file_selector[0]);
+	gtk_container_remove (GTK_CONTAINER (mw->priv->paned),
+						  mw->priv->file_selector[1]);
+
+	gtk_object_destroy (GTK_OBJECT (mw->priv->paned));
+
+	if (gnome_cmd_data_get_list_orientation ())
+		mw->priv->paned = gtk_vpaned_new ();
+	else
+		mw->priv->paned = gtk_hpaned_new ();
+
+	gtk_widget_ref (mw->priv->paned);
+	gtk_object_set_data_full (GTK_OBJECT (main_win), "paned", mw->priv->paned,
+							  (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (mw->priv->paned);
+
+	gtk_paned_pack1 (GTK_PANED (mw->priv->paned),
+					 mw->priv->file_selector[0], TRUE, TRUE);
+	gtk_paned_pack2 (GTK_PANED (mw->priv->paned),
+					 mw->priv->file_selector[1], TRUE, TRUE);
+
+	if (gnome_cmd_data_get_toolbar_visibility ())
+		pos += 2;
+	
+	gtk_box_pack_start (GTK_BOX (mw->priv->vbox), mw->priv->paned, TRUE, TRUE, 0);
+	gtk_box_reorder_child (GTK_BOX (mw->priv->vbox), mw->priv->paned, pos);
+
+	gtk_widget_unref (mw->priv->file_selector[0]);
+	gtk_widget_unref (mw->priv->file_selector[1]);
+
+	gtk_signal_connect (
+		GTK_OBJECT (mw->priv->paned),
+		"button_press_event", GTK_SIGNAL_FUNC (on_slide_button_press), mw);	
+	slide_set_50_50 (NULL, NULL);
+}
+
+
+void
+gnome_cmd_main_win_add_plugin_menu (GnomeCmdMainWin *mw, PluginData *data)
+{
+	g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
+
+	gnome_cmd_main_menu_add_plugin_menu (
+		GNOME_CMD_MAIN_MENU (mw->priv->menubar), data);
+}
+
+
+GnomeCmdState *
+gnome_cmd_main_win_get_state (GnomeCmdMainWin *mw)
+{
+	GnomeCmdState *state;
+	GnomeCmdDir *dir1, *dir2;
+	GnomeCmdFileSelector *fs1, *fs2;
+		
+	g_return_val_if_fail (GNOME_CMD_IS_MAIN_WIN (mw), NULL);
+
+	fs1 = gnome_cmd_main_win_get_active_fs (main_win);
+	fs2 = gnome_cmd_main_win_get_inactive_fs (main_win);
+	dir1 = gnome_cmd_file_selector_get_directory (fs1);
+	dir2 = gnome_cmd_file_selector_get_directory (fs2);
+
+	state = &mw->priv->state;
+	state->active_dir_uri = 
+		gnome_cmd_file_get_uri (GNOME_CMD_FILE (dir1));
+	state->inactive_dir_uri = 
+		gnome_cmd_file_get_uri (GNOME_CMD_FILE (dir2));
+	state->active_dir_files = 
+		gnome_cmd_file_list_get_all_files (fs1->list);
+	state->inactive_dir_files = 
+		gnome_cmd_file_list_get_all_files (fs2->list);
+	state->active_dir_selected_files = 
+		gnome_cmd_file_list_get_selected_files (fs1->list);
+	state->inactive_dir_selected_files =
+		gnome_cmd_file_list_get_selected_files (fs2->list);
+	
+	return state;
+}
+
+
+void
+gnome_cmd_main_win_set_cap_state (GnomeCmdMainWin *mw, gboolean state)
+{
+	g_return_if_fail (GNOME_CMD_IS_MAIN_WIN (mw));
+
+	gtk_widget_set_sensitive (mw->priv->tb_cap_paste_btn, state);
+}
 
