@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2003 Marcus Bjurman
+    Copyright (C) 2001-2006 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,10 +20,10 @@
 #define __GNOME_CMD_PREPARE_MOVE_DIALOG_H__
 
 #include "gnome-cmd-file-list.h"
-#include "gnome-cmd-file-selector-types.h"
+#include "gnome-cmd-file-selector.h"
 
 void
-gnome_cmd_prepare_move_dialog_show (GnomeCmdFileList *from,
+gnome_cmd_prepare_move_dialog_show (GnomeCmdFileSelector *from,
 									GnomeCmdFileSelector *to);
 
 #endif //__GNOME_CMD_PREPARE_MOVE_DIALOG_H__
