@@ -1,6 +1,6 @@
 /*
     GNOME Commander - A GNOME based file manager 
-    Copyright (C) 2001-2003 Marcus Bjurman
+    Copyright (C) 2001-2006 Marcus Bjurman
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,37 +19,38 @@
 #include <config.h>
 #include <sys/types.h>
 #include <regex.h>
-#include <pthread.h>
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-search-dialog.h"
-#include "gnome-cmd-dir-funcs.h"
-#include "gnome-cmd-file-funcs.h"
+#include "gnome-cmd-dir.h"
 #include "gnome-cmd-file-list.h"
-#include "gnome-cmd-file-selector-funcs.h"
+#include "gnome-cmd-file-selector.h"
 #include "gnome-cmd-data.h"
-#include "gnome-cmd-main-win-funcs.h"
-#include "widget-factory.h"
+#include "gnome-cmd-main-win.h"
 #include "filter.h"
-#include "dir_pool.h"
 #include "utils.h"
 
-#define PATTERN_HISTORY_SIZE 10
-
-static GnomeDialogClass *parent_class = NULL;
+static GnomeCmdDialogClass *parent_class = NULL;
 
 #define PBAR_MAX 50
 
+typedef struct
+{
+	GList *files;
+	gchar *msg;
+	GMutex *mutex;
+	
+} ProtectedData;
 
 typedef struct
 {
 	/* the pattern that filenames should match to end up in the file-list */
-	gchar *name_pattern;
+	const gchar *name_pattern;
 
-	/* the pattern that the content of a file should match to end up int the file-list. */
-	gchar *content_pattern;
+	/* the pattern that the content of a file should match to end up in the file-list. */
+	const gchar *content_pattern;
 
 	/* the current dir of the search routine. */
-	gchar *dir;                    
+	const gchar *dir;                    
 
 	Filter *name_filter;
 	regex_t *content_regex;
@@ -81,9 +82,14 @@ typedef struct
 
 	/* The directory to start searching from */
 	GnomeCmdDir *start_dir;
-	GnomeVFSURI *start_dir_uri;
 
-	pthread_t thread;
+	GThread *thread;
+
+	ProtectedData pdata;
+
+	gboolean search_done;
+
+	gint update_gui_timeout_id;
 
 } SearchData;
 
@@ -91,7 +97,7 @@ struct _GnomeCmdSearchDialogPrivate
 {
 	SearchData *data;              // holds data needed by the search routines
 
-	GnomeVFSURI *baseuri;
+	GnomeCmdCon *con;
 	GtkWidget *pattern_combo;
 	GtkWidget *pattern_entry;
 	GtkWidget *dir_browser;
@@ -129,6 +135,17 @@ on_pattern_entry_keypressed (GtkWidget            *entry,
 }
 
 
+static void
+on_list_file_clicked (GnomeCmdFileList *fl,
+					  GnomeCmdFile *finfo,
+					  GdkEventButton *button,
+					  GnomeCmdSearchDialog *dialog)
+{
+	if (dialog->priv->data->search_done)
+		gtk_widget_set_sensitive (dialog->priv->goto_button, TRUE);
+}
+
+
 /**
  * Puts a string in the statusbar.
  *
@@ -136,6 +153,8 @@ on_pattern_entry_keypressed (GtkWidget            *entry,
 static void
 set_statusmsg (SearchData *data, gchar *msg)
 {
+	if (!msg) return;
+	
 	gtk_statusbar_push (GTK_STATUSBAR (data->dialog->priv->statusbar),
 						data->context_id, msg);
 }
@@ -143,40 +162,46 @@ set_statusmsg (SearchData *data, gchar *msg)
 
 /**
  * Loads a file and returns the content.
- * FIXME: Do this with GnomeVFS calls instead.
+ * FIXME: Recode this so not the whole file has
+ *        to be loaded in memory.
  */
 static gchar*
 load_file (GnomeCmdFile *finfo)
 {
-	gchar *buf;
-	const gchar *path;
-	FILE* fd;
-	gint ret;
+	GnomeVFSFileSize ret;
+	gchar *buf, *uri_str;
+	GnomeVFSResult result;
+	GnomeVFSHandle *handle;
 	
 	g_return_val_if_fail (finfo != NULL, NULL);	
 	g_return_val_if_fail (finfo->info != NULL, NULL);
-	
+
 	buf = malloc (finfo->info->size+1);
-	path = gnome_cmd_file_get_path (finfo);	
-	fd = fopen (path, "r");
+	uri_str = gnome_cmd_file_get_uri_str (finfo);
+	result = gnome_vfs_open (&handle, uri_str, GNOME_VFS_OPEN_READ);
 
-	if (fd == NULL)
-	{
-		warn_print (_("Failed to open file %s\n"), path);
+	if (result != GNOME_VFS_OK) {
+		warn_print (_("Failed to open file %s: %s\n"),
+					uri_str, gnome_vfs_result_to_string (result));
+		g_free (uri_str);
+		g_free (buf);
 		return NULL;
 	}
 
-	ret = fread ((void*)buf, 1, finfo->info->size, fd);
-	if (ret != finfo->info->size)
-	{
-		warn_print (_("Failed to read file %s\n"), finfo->path);
-		fclose (fd);
+	result = gnome_vfs_read (handle, buf, finfo->info->size, &ret);
+	if (result != GNOME_VFS_OK) {
+		warn_print (_("Failed to read from file %s: %s\n"),
+					uri_str, gnome_vfs_result_to_string (result));
+		g_free (uri_str);
+		g_free (buf);
+		gnome_vfs_close (handle);
 		return NULL;
 	}
-
-	buf[finfo->info->size] = '\0';
 	
-	fclose (fd);
+	buf[ret] = '\0';
+
+	g_free (uri_str);
+	gnome_vfs_close (handle);
 
 	return buf;
 }
@@ -216,31 +241,31 @@ content_matches (GnomeCmdFile *finfo, SearchData *data)
 static void
 search_dir_r (GnomeCmdDir *dir, SearchData *data)
 {
-	GList *tmp, *list;
-	gchar *msg;
-	const gchar *path;
+	GList *tmp, *files;
+	gchar *path;
 
 	
-	/* update the statusbar and progress bar */
-	gdk_threads_enter ();
+	/* update the search status data */
 	if (!data->dialog_destroyed) {
-		path = gnome_cmd_dir_get_path (dir);
-		msg = g_strdup_printf (_("Searching in: %s"), path);
-		set_statusmsg (data, msg);
-		g_free (msg);	
+		g_mutex_lock (data->pdata.mutex);
+		
+		path = gnome_cmd_file_get_path (GNOME_CMD_FILE (dir));
+		if (data->pdata.msg)
+			g_free (data->pdata.msg);
+		data->pdata.msg = g_strdup_printf (_("Searching in: %s"), path);
+		g_free (path);
 
-		progress_bar_update (data->dialog->priv->pbar, PBAR_MAX);
+		g_mutex_unlock (data->pdata.mutex);
 	}
-	gdk_threads_leave ();
 		
 	
 	/* If the stopbutton was pressed let's abort here */
 	if (data->stopped)
 		return;
-
+	
 	gnome_cmd_dir_list_files (dir, FALSE);
-	gnome_cmd_dir_get_files (dir, &list);
-	tmp = list;
+	gnome_cmd_dir_get_files (dir, &files);
+	tmp = files;
 	
 	/* Let's iterate through all files */
 	while (tmp)
@@ -252,19 +277,20 @@ search_dir_r (GnomeCmdDir *dir, SearchData *data)
 			return;
 
 		/* If the current file is a directory lets continue our recursion */
-		if (finfo->info->type == GNOME_VFS_FILE_TYPE_DIRECTORY && data->recurse)
+		if (GNOME_CMD_IS_DIR (finfo) && data->recurse)
 		{
 			/* we dont want to go backwards or follow symlinks */
 			if (strcmp (finfo->info->name, ".") != 0
 				&& strcmp (finfo->info->name, "..") != 0
 				&& finfo->info->flags != GNOME_VFS_FILE_FLAGS_SYMLINK)
 			{
-				const gchar *dir_uri_str = gnome_cmd_file_get_uri_str (finfo);
-				GnomeCmdDir *new_dir =
-					gnome_cmd_dir_new_from_uri_str (dir_uri_str, FALSE);
-				gnome_cmd_dir_ref (new_dir);
-				search_dir_r (new_dir, data);
-				gnome_cmd_dir_unref (new_dir);
+				GnomeCmdDir *new_dir = GNOME_CMD_DIR (finfo);
+
+				if (new_dir) {
+					gnome_cmd_dir_ref (new_dir);
+					search_dir_r (new_dir, data);
+					gnome_cmd_dir_unref (new_dir);
+				}
 				
 				/* If the stopbutton was pressed let's abort here */
 				if (data->stopped)
@@ -286,12 +312,9 @@ search_dir_r (GnomeCmdDir *dir, SearchData *data)
 			}
 
 			/* the file matched the search criteria, lets add it to the list */			
-			gdk_threads_enter ();
-			if (!data->dialog_destroyed) {
-				gnome_cmd_file_list_add_file (
-					GNOME_CMD_FILE_LIST (data->dialog->priv->result_list), finfo, -1);
-			}
-			gdk_threads_leave ();
+			g_mutex_lock (data->pdata.mutex);
+			data->pdata.files = g_list_append (data->pdata.files, finfo);
+			g_mutex_unlock (data->pdata.mutex);
 
 			/* also ref each directory that has a matching file */
 			if (g_list_index (data->match_dirs, dir) == -1) {
@@ -308,7 +331,7 @@ search_dir_r (GnomeCmdDir *dir, SearchData *data)
 }
 
 
-static void
+static gpointer
 perform_search_operation (SearchData *data)
 {
 	/* Unref all directories which contained matching files from last search */
@@ -318,16 +341,8 @@ perform_search_operation (SearchData *data)
 		data->match_dirs = NULL;
 	}
 	
-	gdk_threads_enter ();
-	gtk_widget_show (data->dialog->priv->pbar);
-	gdk_threads_leave ();
-	
 	search_dir_r (data->start_dir, data);
 	
-	gdk_threads_enter ();
-	gtk_widget_hide (data->dialog->priv->pbar);
-	gdk_threads_leave ();
-
 	/* free regexps */
 	filter_free (data->name_filter);
 	data->name_filter = NULL;
@@ -340,75 +355,145 @@ perform_search_operation (SearchData *data)
 
 	gnome_cmd_dir_unref (data->start_dir);
 	data->start_dir = NULL;
-	gnome_vfs_uri_unref (data->start_dir_uri);
-	data->start_dir_uri = NULL;
-	g_free (data->dir);
 	data->dir = NULL;	
 
-	/* Update the GUI */
-	gdk_threads_enter ();
-	if (!data->dialog_destroyed) {
-		gchar *msg;
-		if (data->stopped)
-			msg = g_strdup_printf (ngettext("Found %d match before I was stopped","Found %d matches before I was stopped",data->matches), data->matches);
-		else
-			msg = g_strdup_printf (ngettext("Found %d match","Found %d matches",data->matches), data->matches);
-		
-		set_statusmsg (data, msg);
-		g_free (msg);
+	data->search_done = TRUE;
+	return NULL;
+}
 
-		gtk_widget_set_sensitive (data->dialog->priv->search_button, TRUE);
-		gtk_widget_set_sensitive (data->dialog->priv->stop_button, FALSE);
+
+static gboolean
+update_search_status_widgets (SearchData *data)
+{
+	GList *files;
+	
+	g_mutex_lock (data->pdata.mutex);
+	
+	/* Add all files found since last update to the list */
+	files = data->pdata.files;
+	while (files) {
+		gnome_cmd_file_list_add_file (
+			GNOME_CMD_FILE_LIST (data->dialog->priv->result_list),
+			GNOME_CMD_FILE (files->data), -1);
+		files = files->next;
 	}
-	gdk_threads_leave ();
+	if (data->pdata.files) {
+		g_list_free (data->pdata.files);
+		data->pdata.files = NULL;
+	}
+
+	/* Update statusbar with the latest message */
+	set_statusmsg (data, data->pdata.msg);
+
+	/* Update the progressbar */
+	progress_bar_update (data->dialog->priv->pbar, PBAR_MAX);
+
+	g_mutex_unlock (data->pdata.mutex);
+	
+	if (data->search_done) {		
+		if (!data->dialog_destroyed) {
+			gchar *msg;
+			if (data->stopped)
+				msg = g_strdup_printf (
+					ngettext(
+						"Found %d match before I was stopped",
+						"Found %d matches before I was stopped",
+						data->matches),
+					data->matches);
+			else
+				msg = g_strdup_printf (
+					ngettext(
+						"Found %d match",
+						"Found %d matches",
+						data->matches),
+					data->matches);
+		
+			set_statusmsg (data, msg);
+			g_free (msg);
+
+			gtk_widget_set_sensitive (data->dialog->priv->search_button, TRUE);
+			gtk_widget_set_sensitive (data->dialog->priv->stop_button, FALSE);
+	
+			gtk_widget_hide (data->dialog->priv->pbar);
+		}
+
+		/* Returning FALSE here stops the timeout callbacks */
+		return FALSE;
+	}
+	
+	return TRUE;
 }
 
 
 /*
- * Tells the thread to exit and then waits for it to do so.
+ * This function gets called then the search-dialog is about the be destroyed.
+ * The function waits for the last search-thread to finish and then frees the
+ * data structure that has been shared between the search threads and the
+ * main thread.
  */
 static gboolean
 join_thread_func (SearchData *data)
 {
-	pthread_join (data->thread, NULL);
+	if (data->thread)
+		g_thread_join (data->thread);
 
-	if (data->dialog_destroyed)
-		g_free (data);
+	g_free (data);
 	
 	return FALSE;
 }
 
 
 static void
-on_search_window_destroy (GnomeCmdSearchDialog *dialog, gpointer user_data)
+on_dialog_destroy (GnomeCmdSearchDialog *dialog, gpointer user_data)
 {
-	if (dialog->priv->data) {
+	SearchData *data = dialog->priv->data;
+
+	if (data) {
+		if (!data->search_done)
+			gtk_timeout_remove (data->update_gui_timeout_id);
 		
 		/* Stop and wait for search thread to exit */
-		dialog->priv->data->stopped = TRUE;
-		dialog->priv->data->dialog_destroyed = TRUE;
-		gtk_timeout_add (1, (GtkFunction)join_thread_func, dialog->priv->data);
+		data->stopped = TRUE;
+		data->dialog_destroyed = TRUE;
+		gtk_timeout_add (1, (GtkFunction)join_thread_func, data);
 
 		/* Unref all directories which contained matching files from last search */
-		if (dialog->priv->data->match_dirs) {
-			g_list_foreach (dialog->priv->data->match_dirs, (GFunc)gnome_cmd_dir_unref, NULL);
-			g_list_free (dialog->priv->data->match_dirs);
-			dialog->priv->data->match_dirs = NULL;
-		}	
+		if (data->pdata.mutex != NULL) {
+			g_mutex_lock (data->pdata.mutex);
+			if (data->match_dirs) {
+				g_list_foreach (data->match_dirs, (GFunc)gnome_cmd_dir_unref, NULL);
+				g_list_free (data->match_dirs);
+				data->match_dirs = NULL;
+			}	
+			g_mutex_unlock (data->pdata.mutex);
+		}
 	}
 }
 
 
-/**
- * The user has clicked on the search button 
- *
- */
 static void
-on_search (GtkButton *button, GnomeCmdSearchDialog *dialog)
+on_dialog_size_allocate (GtkWidget       *widget,
+						 GtkAllocation   *allocation,
+						 GnomeCmdSearchDialog *dialog)
 {
+	SearchDefaults *defaults = gnome_cmd_data_get_search_defaults ();
+	defaults->width  = allocation->width;
+	defaults->height = allocation->height;
+}
+
+
+static gboolean
+start_search (GnomeCmdSearchDialog *dialog)
+{
+	GnomeCmdPath *path;
 	SearchData *data = dialog->priv->data;
 	SearchDefaults *defaults = gnome_cmd_data_get_search_defaults ();
 
+	if (data->thread) {
+		g_thread_join (data->thread);
+		data->thread = NULL;
+	}
+	
 	data->dialog = dialog;
 	data->name_pattern = gtk_entry_get_text (GTK_ENTRY (dialog->priv->pattern_entry));
 	data->content_pattern = gtk_entry_get_text (GTK_ENTRY (dialog->priv->find_text_entry));
@@ -420,7 +505,6 @@ on_search (GtkButton *button, GnomeCmdSearchDialog *dialog)
 	data->match_dirs = NULL;
 	data->stopped = FALSE;
 	data->dialog_destroyed = FALSE;
-	data->thread = 0;
 	data->recurse = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->recurse_check));
 	data->case_sens = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->priv->case_check));
 
@@ -453,13 +537,35 @@ on_search (GtkButton *button, GnomeCmdSearchDialog *dialog)
 
 	
 	/* start the search */
-	data->start_dir_uri = gnome_vfs_uri_append_path (dialog->priv->baseuri, data->dir);
-	data->dir = gnome_vfs_uri_to_string (data->start_dir_uri, 0);
-	data->start_dir = dir_pool_get (data->dir);
+	path = gnome_cmd_con_create_path (dialog->priv->con, data->dir);
+	data->start_dir = gnome_cmd_dir_new (dialog->priv->con, path);
 	gnome_cmd_dir_ref (data->start_dir);
-
 	
-	pthread_create (&data->thread, NULL, (PthreadFunc)perform_search_operation, data);
+	data->search_done = FALSE;
+	data->pdata.mutex = g_mutex_new ();
+	data->thread = g_thread_create ((GThreadFunc)perform_search_operation, data, TRUE, NULL);
+
+	gtk_widget_show (data->dialog->priv->pbar);
+	data->update_gui_timeout_id = gtk_timeout_add (
+		gnome_cmd_data_get_gui_update_rate (),
+		(GtkFunction)update_search_status_widgets,
+		dialog->priv->data);
+
+	return FALSE;
+}
+
+
+/**
+ * The user has clicked on the search button 
+ *
+ */
+static void
+on_search (GtkButton *button, GnomeCmdSearchDialog *dialog)
+{
+	gtk_timeout_add (1, (GtkFunction)start_search, dialog);
+
+	gtk_widget_set_sensitive (dialog->priv->search_button, FALSE);
+	gtk_widget_set_sensitive (dialog->priv->goto_button, FALSE);
 }
 
 
@@ -486,7 +592,8 @@ on_stop (GtkButton *button, GnomeCmdSearchDialog *dialog)
 	g_return_if_fail (dialog->priv->data != NULL);
 	
 	dialog->priv->data->stopped = TRUE;
-	gtk_timeout_add (1, (GtkFunction)join_thread_func, dialog->priv->data);
+
+	gtk_widget_set_sensitive (dialog->priv->stop_button, FALSE);
 }
 
 
@@ -499,17 +606,22 @@ on_goto (GtkButton *button, GnomeCmdSearchDialog *dialog)
 {
 	GnomeCmdFile *finfo;
 	GnomeCmdFileSelector *fs;
+	gchar *dpath, *fpath;
 	
 	finfo = gnome_cmd_file_list_get_selected_file (GNOME_CMD_FILE_LIST (dialog->priv->result_list));
-	if (!finfo) {
-		g_printerr (_("No file selected\n"));
+	if (!finfo)
 		return;
-	}
+
+	fpath = gnome_cmd_file_get_path (finfo);
+	dpath = g_dirname (fpath);
 
 	fs = gnome_cmd_main_win_get_active_fs (main_win);
-	gnome_cmd_file_selector_goto_directory (fs, gnome_cmd_file_get_dir_path (finfo));
+	gnome_cmd_file_selector_goto_directory (fs, dpath);
 	gnome_cmd_file_list_focus_file (fs->list, gnome_cmd_file_get_name (finfo), TRUE);
 
+	g_free (fpath);
+	g_free (dpath);
+	
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 }
 
@@ -591,7 +703,7 @@ class_init (GnomeCmdSearchDialogClass *class)
 	object_class = GTK_OBJECT_CLASS (class);
 	widget_class = GTK_WIDGET_CLASS (class);
 
-	parent_class = gtk_type_class (gnome_dialog_get_type ());
+	parent_class = gtk_type_class (gnome_cmd_dialog_get_type ());
 
 	object_class->destroy = destroy;
 
@@ -605,35 +717,35 @@ init (GnomeCmdSearchDialog *dialog)
 	GtkWidget *vbox1;
 	GtkWidget *table;
 	GtkWidget *label;
-	GtkWidget *search_button;
-	GtkWidget *close_button;
-	GtkWidget *stop_button;
 	GtkWidget *sw;
 	GtkWidget *pbar;
-	GList     *buttons;
 	SearchDefaults *defaults = gnome_cmd_data_get_search_defaults ();
 	
 
 	dialog->priv = g_new (GnomeCmdSearchDialogPrivate, 1);
 	dialog->priv->data = g_new (SearchData, 1);
 	dialog->priv->data->match_dirs = NULL;
+	dialog->priv->data->thread = NULL;
+	dialog->priv->data->search_done = TRUE;
 
+	dialog->priv->data->pdata.files = NULL;
+	dialog->priv->data->pdata.msg = NULL;
+	dialog->priv->data->pdata.mutex = NULL;
 	
 	window = GTK_WIDGET (dialog);
 	gtk_object_set_data (GTK_OBJECT (window), "window", window);
 	gtk_window_set_title (GTK_WINDOW (window), _("Search..."));
-	gtk_window_set_policy (GTK_WINDOW (window), FALSE, TRUE, FALSE);
-	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
+	gnome_cmd_dialog_set_resizable (GNOME_CMD_DIALOG (dialog), TRUE);
+	gtk_window_set_default_size (
+		GTK_WINDOW (window), defaults->width, defaults->height);
 	
 
-	vbox1 = GNOME_DIALOG (window)->vbox;
+	vbox1 = create_vbox (window, FALSE, 0);
+	gnome_cmd_dialog_add_expanding_category (GNOME_CMD_DIALOG (dialog), vbox1);
 
-	/*
-	 * The "Basic" tab
-	 */
 	table = create_table (window, 5, 2);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 5);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 2);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
 	gtk_box_pack_start (GTK_BOX (vbox1), table, FALSE, TRUE, 0);
 
 	/* Search for */
@@ -647,7 +759,8 @@ init (GnomeCmdSearchDialog *dialog)
 									   defaults->name_patterns);
 
 	dialog->priv->pattern_entry = GTK_COMBO (dialog->priv->pattern_combo)->entry;
-    gnome_dialog_editable_enters (GNOME_DIALOG (dialog), GTK_EDITABLE (dialog->priv->pattern_entry));
+    gnome_cmd_dialog_editable_enters (
+		GNOME_CMD_DIALOG (dialog), GTK_EDITABLE (dialog->priv->pattern_entry));
 
 	/* Search in */
 	label = create_label (window, _("Search in: "));
@@ -657,7 +770,7 @@ init (GnomeCmdSearchDialog *dialog)
 	table_add (table, dialog->priv->dir_browser, 1, 1, GTK_EXPAND|GTK_FILL);
 	if (defaults->directories)
 		gtk_combo_set_popdown_strings (
-			GTK_COMBO (GNOME_FILE_ENTRY (dialog->priv->dir_browser)->gentry),
+			GTK_COMBO (gnome_file_entry_gnome_entry (GNOME_FILE_ENTRY (dialog->priv->dir_browser))),
 			defaults->directories);
 
 	dialog->priv->dir_entry =
@@ -675,7 +788,8 @@ init (GnomeCmdSearchDialog *dialog)
 									   defaults->content_patterns);
 
 	dialog->priv->find_text_entry = GTK_COMBO (dialog->priv->find_text_combo)->entry;
-    gnome_dialog_editable_enters (GNOME_DIALOG (dialog), GTK_EDITABLE (dialog->priv->find_text_entry));
+    gnome_cmd_dialog_editable_enters (
+		GNOME_CMD_DIALOG (dialog), GTK_EDITABLE (dialog->priv->find_text_entry));
 
 
 	/* Recurse check */
@@ -696,31 +810,22 @@ init (GnomeCmdSearchDialog *dialog)
 								  defaults->case_sens);
 
 
-	
-	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window), _("Search"),
-											GNOME_STOCK_PIXMAP_SEARCH);
-	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window), _("Stop"),
-											GNOME_STOCK_PIXMAP_STOP);
-	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window), _("Goto"),
-											GNOME_STOCK_PIXMAP_JUMP_TO);
-	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window), _("Close"),
-											GNOME_STOCK_PIXMAP_CLOSE);
-	
+	dialog->priv->close_button = gnome_cmd_dialog_add_button (
+		GNOME_CMD_DIALOG (dialog), GNOME_STOCK_PIXMAP_CLOSE,
+		GTK_SIGNAL_FUNC (on_close), dialog);
+	dialog->priv->goto_button = gnome_cmd_dialog_add_button (
+		GNOME_CMD_DIALOG (dialog), _("Goto"),
+		GTK_SIGNAL_FUNC (on_goto), dialog);
+	dialog->priv->stop_button = gnome_cmd_dialog_add_button (
+		GNOME_CMD_DIALOG (dialog), GNOME_STOCK_PIXMAP_STOP,
+		GTK_SIGNAL_FUNC (on_stop), dialog);
+	dialog->priv->search_button = gnome_cmd_dialog_add_button (
+		GNOME_CMD_DIALOG (dialog), GNOME_STOCK_PIXMAP_SEARCH,
+		GTK_SIGNAL_FUNC (on_search), dialog);
 
-	buttons = g_list_first (GNOME_DIALOG (window)->buttons);
-	search_button = GTK_WIDGET (buttons->data);
-	dialog->priv->search_button = search_button;
-	buttons = buttons->next;
-	stop_button = GTK_WIDGET (buttons->data);
-	gtk_widget_set_sensitive (stop_button, FALSE);
-	dialog->priv->stop_button = stop_button;
-	buttons = buttons->next;
-	dialog->priv->goto_button = GTK_WIDGET (buttons->data);
-	buttons = buttons->next;
-	close_button = GTK_WIDGET (buttons->data);
-	dialog->priv->close_button = close_button;
+	gtk_widget_set_sensitive (dialog->priv->stop_button, FALSE);
+	gtk_widget_set_sensitive (dialog->priv->goto_button, FALSE);
 	
-	gnome_dialog_set_default (GNOME_DIALOG (dialog), 0);
 	
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_widget_ref (sw);
@@ -744,12 +849,13 @@ init (GnomeCmdSearchDialog *dialog)
 
 
 	dialog->priv->statusbar = gtk_statusbar_new ();
+	gtk_statusbar_set_has_resize_grip (
+		GTK_STATUSBAR (dialog->priv->statusbar), FALSE);
 	gtk_widget_ref (dialog->priv->statusbar);
 	gtk_object_set_data_full (GTK_OBJECT (window), "statusbar", dialog->priv->statusbar,
 							  (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (dialog->priv->statusbar);
 	gtk_box_pack_start (GTK_BOX (vbox1), dialog->priv->statusbar, FALSE, TRUE, 0);
-	gtk_widget_set_usize (dialog->priv->statusbar, 500, -1);
 
 	pbar = create_progress_bar (window);
 	gtk_widget_hide (pbar);
@@ -761,17 +867,13 @@ init (GnomeCmdSearchDialog *dialog)
 
 
 	gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
-						GTK_SIGNAL_FUNC (on_search_window_destroy), NULL);
+						GTK_SIGNAL_FUNC (on_dialog_destroy), NULL);
+	gtk_signal_connect (GTK_OBJECT (dialog), "size-allocate",
+						GTK_SIGNAL_FUNC (on_dialog_size_allocate), NULL);
 	gtk_signal_connect (GTK_OBJECT (dialog->priv->result_list), "key-press-event",
 						GTK_SIGNAL_FUNC (on_list_keypressed), dialog);
-	gtk_signal_connect (GTK_OBJECT (search_button), "clicked",
-						GTK_SIGNAL_FUNC (on_search), dialog);
-	gtk_signal_connect (GTK_OBJECT (stop_button), "clicked",
-						GTK_SIGNAL_FUNC (on_stop), dialog);
-	gtk_signal_connect (GTK_OBJECT (dialog->priv->goto_button), "clicked",
-						GTK_SIGNAL_FUNC (on_goto), dialog);
-	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
-						GTK_SIGNAL_FUNC (on_close), dialog);
+	gtk_signal_connect (GTK_OBJECT (dialog->priv->result_list), "file-clicked",
+						GTK_SIGNAL_FUNC (on_list_file_clicked), dialog);
 
 	gtk_signal_connect (GTK_OBJECT (dialog->priv->pattern_entry),
 						"key-press-event",
@@ -811,33 +913,35 @@ gnome_cmd_search_dialog_get_type         (void)
 			(GtkClassInitFunc) NULL
 		};
 
-		dlg_type = gtk_type_unique (gnome_dialog_get_type (), &dlg_info);
+		dlg_type = gtk_type_unique (gnome_cmd_dialog_get_type (), &dlg_info);
 	}
 	return dlg_type;
 }
 
 
 GtkWidget*
-gnome_cmd_search_dialog_new (GnomeVFSURI *baseuri, GnomeCmdDir *default_dir)
+gnome_cmd_search_dialog_new (GnomeCmdDir *default_dir)
 {
-	const gchar *path;
+	gchar *path;
 	gchar *new_path;
 	GnomeCmdSearchDialog *dialog;
 
 	dialog = gtk_type_new (gnome_cmd_search_dialog_get_type ());
 
-	path = gnome_cmd_dir_get_path (default_dir);
-	if (path[strlen(path)-1] != '/')
+	path = gnome_cmd_file_get_path (GNOME_CMD_FILE (default_dir));
+	if (path[strlen(path)-1] != '/') {
 		new_path = g_strdup_printf ("%s/", path);
+		g_free (path);
+	}
 	else
-		new_path = g_strdup (path);
+		new_path = path;
 	
 	gtk_entry_set_text (
 		GTK_ENTRY (dialog->priv->dir_entry), new_path);
 
 	g_free (new_path);
 
-	dialog->priv->baseuri = baseuri;
+	dialog->priv->con = gnome_cmd_dir_get_connection (default_dir);
 
 	return GTK_WIDGET (dialog);
 }
